@@ -1,10 +1,10 @@
 /******************************************************************************
  *
- * Copyright (C) 1997-2006 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
- * documentation under the terms of the GNU General Public License is hereby 
- * granted. No representations are made about the suitability of this software 
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
  * for any purpose. It is provided "as is" without express or implied warranty.
  * See the GNU General Public License for more details.
  *
@@ -19,17 +19,25 @@
  *  This example shows how to configure and run doxygen programmatically from
  *  within an application without generating the usual output.
  *  The example should work on any Unix like OS (including Linux and Mac OS X).
- *  
+ *
  *  This example shows how to use to code parser to get cross-references information
  *  and it also shows how to look up symbols in a program parsed by doxygen and
  *  show some information about them.
  */
 
 #include <stdlib.h>
-#include <unistd.h>
+#include <qfile.h>
+#include <qdir.h>
 #include "doxygen.h"
 #include "outputgen.h"
 #include "parserintf.h"
+#include "classdef.h"
+#include "namespacedef.h"
+#include "filedef.h"
+#include "util.h"
+#include "classlist.h"
+#include "config.h"
+#include "filename.h"
 
 class XRefDummyCodeGenerator : public CodeOutputInterface
 {
@@ -41,17 +49,22 @@ class XRefDummyCodeGenerator : public CodeOutputInterface
     // and cross-linked version of the source code, but who needs that anyway ;-)
     void codify(const char *) {}
     void writeCodeLink(const char *,const char *,const char *,const char *,const char *)  {}
-    void startCodeLine() {}
+    void writeLineNumber(const char *,const char *,const char *,int) {}
+    virtual void writeTooltip(const char *,const DocLinkInfo &,
+                              const char *,const char *,const SourceLinkInfo &,
+                              const SourceLinkInfo &) {}
+    void startCodeLine(bool) {}
     void endCodeLine() {}
     void startCodeAnchor(const char *) {}
     void endCodeAnchor() {}
     void startFontClass(const char *) {}
     void endFontClass() {}
     void writeCodeAnchor(const char *) {}
-    void writeLineNumber(const char *,const char *,const char *,int) {}
+    void setCurrentDoc(const Definition *,const char *,bool) {}
+    void addWord(const char *,bool) {}
 
     // here we are presented with the symbols found by the code parser
-    void linkableSymbol(int l, const char *sym,Definition *symDef,Definition *context) 
+    void linkableSymbol(int l, const char *sym,Definition *symDef,Definition *context)
     {
       QCString ctx;
       if (context) // the context of the symbol is known
@@ -63,15 +76,15 @@ class XRefDummyCodeGenerator : public CodeOutputInterface
              // it is inside a member of a class
           {
             ctx.sprintf("inside %s %s of %s %s",
-              ((MemberDef *)context)->memberTypeName().data(),
+              (dynamic_cast<MemberDef*>(context))->memberTypeName().data(),
               context->name().data(),
-              ((ClassDef*)parentContext)->compoundTypeString().data(),
+              (dynamic_cast<ClassDef*>(parentContext))->compoundTypeString().data(),
               parentContext->name().data());
           }
           else if (parentContext==Doxygen::globalScope) // it is inside a global member
           {
             ctx.sprintf("inside %s %s",
-              ((MemberDef *)context)->memberTypeName().data(),
+              (dynamic_cast<MemberDef*>(context))->memberTypeName().data(),
               context->name().data());
           }
         }
@@ -96,18 +109,22 @@ class XRefDummyCodeGenerator : public CodeOutputInterface
 static void findXRefSymbols(FileDef *fd)
 {
   // get the interface to a parser that matches the file extension
-  ParserInterface *pIntf=Doxygen::parserManager->getParser(fd->getDefFileExtension());
+  CodeParserInterface &intf=Doxygen::parserManager->getCodeParser(fd->getDefFileExtension());
+
+  // get the programming language from the file name
+  SrcLangExt lang = getLanguageFromFileName(fd->name());
 
   // reset the parsers state
-  pIntf->resetCodeParserState();
+  intf.resetCodeParserState();
 
-  // create a new backend object 
+  // create a new backend object
   XRefDummyCodeGenerator *xrefGen = new XRefDummyCodeGenerator(fd);
 
   // parse the source code
-  pIntf->parseCode(*xrefGen,
+  intf.parseCode(*xrefGen,
                 0,
                 fileToString(fd->absFilePath()),
+                lang,
                 FALSE,
                 0,
                 fd);
@@ -120,7 +137,7 @@ static void listSymbol(Definition *d)
 {
   if (d!=Doxygen::globalScope && // skip the global namespace symbol
       d->name().at(0)!='@'       // skip anonymous stuff
-     )      
+     )
   {
     printf("%s\n",
         d->name().data());
@@ -155,7 +172,7 @@ static void lookupSymbol(Definition *d)
 {
   if (d!=Doxygen::globalScope && // skip the global namespace symbol
       d->name().at(0)!='@'       // skip anonymous stuff
-     )      
+     )
   {
     printf("Symbol info\n");
     printf("-----------\n");
@@ -168,20 +185,20 @@ static void lookupSymbol(Definition *d)
     {
       case Definition::TypeClass:
         {
-          ClassDef *cd = (ClassDef *)d;
+          ClassDef *cd = dynamic_cast<ClassDef*>(d);
           printf("Kind: %s\n",cd->compoundTypeString().data());
         }
         break;
       case Definition::TypeFile:
         {
-          FileDef *fd = (FileDef *)d;
+          FileDef *fd = dynamic_cast<FileDef*>(d);
           printf("Kind: File: #includes %d other files\n",
               fd->includeFileList() ? fd->includeFileList()->count() : 0);
         }
         break;
       case Definition::TypeNamespace:
         {
-          NamespaceDef *nd = (NamespaceDef *)d;
+          NamespaceDef *nd = dynamic_cast<NamespaceDef*>(d);
           printf("Kind: Namespace: contains %d classes and %d namespaces\n",
               nd->getClassSDict() ? nd->getClassSDict()->count() : 0,
               nd->getNamespaceSDict() ? nd->getNamespaceSDict()->count() : 0);
@@ -189,7 +206,7 @@ static void lookupSymbol(Definition *d)
         break;
       case Definition::TypeMember:
         {
-          MemberDef *md = (MemberDef *)d;
+          MemberDef *md = dynamic_cast<MemberDef*>(d);
           printf("Kind: %s\n",md->memberTypeName().data());
         }
         break;
@@ -239,62 +256,58 @@ int main(int argc,char **argv)
     exit(1);
   }
 
-  // initialize data structures 
+  // initialize data structures
   initDoxygen();
 
   // setup the non-default configuration options
 
+  checkConfiguration();
+  adjustConfiguration();
   // we need a place to put intermediate files
-  Config_getString("OUTPUT_DIRECTORY")="/tmp/doxygen"; 
+  Config_getString(OUTPUT_DIRECTORY)="/tmp/doxygen";
   // disable html output
-  Config_getBool("GENERATE_HTML")=FALSE;
+  Config_getBool(GENERATE_HTML)=FALSE;
   // disable latex output
-  Config_getBool("GENERATE_LATEX")=FALSE;
+  Config_getBool(GENERATE_LATEX)=FALSE;
   // be quiet
-  Config_getBool("QUIET")=TRUE;
+  Config_getBool(QUIET)=TRUE;
   // turn off warnings
-  Config_getBool("WARNINGS")=FALSE;
-  Config_getBool("WARN_IF_UNDOCUMENTED")=FALSE;
-  Config_getBool("WARN_IF_DOC_ERROR")=FALSE;
+  Config_getBool(WARNINGS)=FALSE;
+  Config_getBool(WARN_IF_UNDOCUMENTED)=FALSE;
+  Config_getBool(WARN_IF_DOC_ERROR)=FALSE;
   // Extract as much as possible
-  Config_getBool("EXTRACT_ALL")=TRUE;
-  Config_getBool("EXTRACT_STATIC")=TRUE;
-  Config_getBool("EXTRACT_PRIVATE")=TRUE;
-  Config_getBool("EXTRACT_LOCAL_METHODS")=TRUE;
-  // Extract source browse information, needed 
+  Config_getBool(EXTRACT_ALL)=TRUE;
+  Config_getBool(EXTRACT_STATIC)=TRUE;
+  Config_getBool(EXTRACT_PRIVATE)=TRUE;
+  Config_getBool(EXTRACT_LOCAL_METHODS)=TRUE;
+  // Extract source browse information, needed
   // to make doxygen gather the cross reference info
-  Config_getBool("SOURCE_BROWSER")=TRUE;
+  Config_getBool(SOURCE_BROWSER)=TRUE;
+  // In case of a directory take all files on directory and its subdirectories
+  Config_getBool(RECURSIVE)=TRUE;
 
   // set the input
-  Config_getList("INPUT").append(argv[1]);
-
-  // check and finialize the configuration
-  checkConfiguration();
+  Config_getList(INPUT).clear();
+  Config_getList(INPUT).append(argv[1]);
 
   // parse the files
   parseInput();
 
   // iterate over the input files
-  FileNameListIterator fnli(*Doxygen::inputNameList); 
-  FileName *fn;
-  // foreach file with a certain name
-  for (fnli.toFirst();(fn=fnli.current());++fnli)
+  for (const auto &fn : *Doxygen::inputNameLinkedMap)
   {
-    FileNameIterator fni(*fn);
-    FileDef *fd;
-    // for each file definition
-    for (;(fd=fni.current());++fni)
+    for (const auto &fd : *fn)
     {
       // get the references (linked and unlinked) found in this file
-      findXRefSymbols(fd);
+      findXRefSymbols(fd.get());
     }
   }
 
   // remove temporary files
-  if (!Doxygen::objDBFileName.isEmpty()) unlink(Doxygen::objDBFileName);
-  if (!Doxygen::entryDBFileName.isEmpty()) unlink(Doxygen::entryDBFileName);
+  if (!Doxygen::objDBFileName.isEmpty()) QFile::remove(Doxygen::objDBFileName);
+  if (!Doxygen::entryDBFileName.isEmpty()) QFile::remove(Doxygen::entryDBFileName);
   // clean up after us
-  rmdir("/tmp/doxygen");
+  QDir().rmdir("/tmp/doxygen");
 
   while (1)
   {
@@ -302,11 +315,11 @@ int main(int argc,char **argv)
     fgets(cmd,256,stdin);
     QCString s(cmd);
     if (s.at(s.length()-1)=='\n') s=s.left(s.length()-1); // strip trailing \n
-    if (s==".list") 
+    if (s==".list")
       listSymbols();
-    else if (s==".quit") 
+    else if (s==".quit")
       exit(0);
-    else 
+    else
       lookupSymbols(s);
   }
 }
