@@ -32,7 +32,7 @@ class BaseClassList;
 class NamespaceSDict;
 class FileDef;
 class FileList;
-class FileNameList;
+class FileNameLinkedMap;
 class DirSDict;
 class DirList;
 class DirDef;
@@ -52,7 +52,10 @@ class MemberGroup;
 class MemberGroupSDict;
 class MemberGroupList;
 class DotNode;
-class DotGfxHierarchyTable; 
+class DotGfxHierarchyTable;
+struct SearchIndexInfo;
+class SearchIndexList;
+class SearchDefinitionList;
 
 //----------------------------------------------------
 
@@ -68,7 +71,7 @@ class RefCountedContext
       m_className=className;
       m_insideRelease = FALSE;
     }
-    ~RefCountedContext()
+    virtual ~RefCountedContext()
     {
       if (!m_insideRelease) abort();
     }
@@ -183,19 +186,19 @@ class TranslateContext : public RefCountedContext, public TemplateStructIntf
 class UsedFilesContext : public RefCountedContext, public TemplateListIntf
 {
   public:
-    static UsedFilesContext *alloc(ClassDef *cd) { return new UsedFilesContext(cd); }
+    static UsedFilesContext *alloc(const ClassDef *cd) { return new UsedFilesContext(cd); }
 
     // TemplateListIntf
-    virtual int count() const;
-    virtual TemplateVariant at(int index) const;
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
     virtual TemplateListIntf::ConstIterator *createIterator() const;
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
 
-    void addFile(FileDef *fd);
+    void addFile(const FileDef *fd);
 
   private:
-    UsedFilesContext(ClassDef *cd);
+    UsedFilesContext(const ClassDef *cd);
    ~UsedFilesContext();
 
     class Private;
@@ -231,8 +234,8 @@ class IncludeInfoListContext : public RefCountedContext, public TemplateListIntf
     { return new IncludeInfoListContext(list,lang); }
 
     // TemplateListIntf
-    virtual int count() const;
-    virtual TemplateVariant at(int index) const;
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
     virtual TemplateListIntf::ConstIterator *createIterator() const;
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
@@ -250,7 +253,7 @@ class IncludeInfoListContext : public RefCountedContext, public TemplateListIntf
 class ClassContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static ClassContext *alloc(ClassDef *cd) { return new ClassContext(cd); }
+    static ClassContext *alloc(const ClassDef *cd) { return new ClassContext(cd); }
 
     // TemplateStructIntf methods
     virtual TemplateVariant get(const char *name) const;
@@ -258,7 +261,7 @@ class ClassContext : public RefCountedContext, public TemplateStructIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    ClassContext(ClassDef *);
+    ClassContext(const ClassDef *);
    ~ClassContext();
     class Private;
     Private *p;
@@ -269,7 +272,7 @@ class ClassContext : public RefCountedContext, public TemplateStructIntf
 class NamespaceContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static NamespaceContext *alloc(NamespaceDef *nd) { return new NamespaceContext(nd); }
+    static NamespaceContext *alloc(const NamespaceDef *nd) { return new NamespaceContext(nd); }
 
     // TemplateStructIntf methods
     virtual TemplateVariant get(const char *name) const;
@@ -277,7 +280,7 @@ class NamespaceContext : public RefCountedContext, public TemplateStructIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    NamespaceContext(NamespaceDef *);
+    NamespaceContext(const NamespaceDef *);
    ~NamespaceContext();
     class Private;
     Private *p;
@@ -288,7 +291,7 @@ class NamespaceContext : public RefCountedContext, public TemplateStructIntf
 class FileContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static FileContext *alloc(FileDef *fd) { return new FileContext(fd); }
+    static FileContext *alloc(const FileDef *fd) { return new FileContext(fd); }
 
     // TemplateStructIntf methods
     virtual TemplateVariant get(const char *name) const;
@@ -296,7 +299,7 @@ class FileContext : public RefCountedContext, public TemplateStructIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    FileContext(FileDef *);
+    FileContext(const FileDef *);
    ~FileContext();
     class Private;
     Private *p;
@@ -306,7 +309,7 @@ class FileContext : public RefCountedContext, public TemplateStructIntf
 class DirContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static DirContext *alloc(DirDef *dd) { return new DirContext(dd); }
+    static DirContext *alloc(const DirDef *dd) { return new DirContext(dd); }
 
     // TemplateStructIntf methods
     virtual TemplateVariant get(const char *name) const;
@@ -314,7 +317,7 @@ class DirContext : public RefCountedContext, public TemplateStructIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    DirContext(DirDef *);
+    DirContext(const DirDef *);
    ~DirContext();
     class Private;
     Private *p;
@@ -326,7 +329,7 @@ class DirContext : public RefCountedContext, public TemplateStructIntf
 class PageContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static PageContext *alloc(PageDef *pd,bool isMainPage=FALSE) { return new PageContext(pd,isMainPage); }
+    static PageContext *alloc(const PageDef *pd,bool isMainPage,bool isExample) { return new PageContext(pd,isMainPage,isExample); }
 
     // TemplateStructIntf methods
     virtual TemplateVariant get(const char *name) const;
@@ -334,7 +337,7 @@ class PageContext : public RefCountedContext, public TemplateStructIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    PageContext(PageDef *,bool isMainPage);
+    PageContext(const PageDef *,bool isMainPage,bool isExample);
    ~PageContext();
     class Private;
     Private *p;
@@ -365,7 +368,7 @@ class MemberContext : public RefCountedContext, public TemplateStructIntf
 class ModuleContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static ModuleContext *alloc(GroupDef *gd) { return new ModuleContext(gd); }
+    static ModuleContext *alloc(const GroupDef *gd) { return new ModuleContext(gd); }
 
     // TemplateStructIntf methods
     virtual TemplateVariant get(const char *name) const;
@@ -373,7 +376,7 @@ class ModuleContext : public RefCountedContext, public TemplateStructIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    ModuleContext(GroupDef *);
+    ModuleContext(const GroupDef *);
    ~ModuleContext();
     class Private;
     Private *p;
@@ -387,8 +390,8 @@ class ClassListContext : public RefCountedContext, public TemplateListIntf
     static ClassListContext *alloc() { return new ClassListContext; }
 
     // TemplateListIntf
-    virtual int  count() const;
-    virtual TemplateVariant at(int index) const;
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
     virtual TemplateListIntf::ConstIterator *createIterator() const;
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
@@ -444,7 +447,7 @@ class InheritanceGraphContext : public RefCountedContext, public TemplateStructI
 class ClassInheritanceNodeContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static ClassInheritanceNodeContext *alloc(ClassDef *cd)
+    static ClassInheritanceNodeContext *alloc(const ClassDef *cd)
     { return new ClassInheritanceNodeContext(cd); }
 
     // TemplateStructIntf methods
@@ -455,7 +458,7 @@ class ClassInheritanceNodeContext : public RefCountedContext, public TemplateStr
     void addChildren(const BaseClassList *bcl,bool hideSuper);
 
   private:
-    ClassInheritanceNodeContext(ClassDef *);
+    ClassInheritanceNodeContext(const ClassDef *);
    ~ClassInheritanceNodeContext();
     class Private;
     Private *p;
@@ -469,8 +472,8 @@ class ClassInheritanceContext : public RefCountedContext, public TemplateListInt
     static ClassInheritanceContext *alloc() { return new ClassInheritanceContext; }
 
     // TemplateListIntf
-    virtual int  count() const;
-    virtual TemplateVariant at(int index) const;
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
     virtual TemplateListIntf::ConstIterator *createIterator() const;
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
@@ -506,7 +509,7 @@ class ClassHierarchyContext : public RefCountedContext, public TemplateStructInt
 class NestingNodeContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static NestingNodeContext *alloc(const NestingNodeContext *parent,Definition *def,
+    static NestingNodeContext *alloc(const NestingNodeContext *parent,const Definition *def,
                                      int index,int level,bool addClasses,bool inherit,bool hideSuper)
     { return new NestingNodeContext(parent,def,index,level,addClasses,inherit,hideSuper); }
 
@@ -519,7 +522,7 @@ class NestingNodeContext : public RefCountedContext, public TemplateStructIntf
 
   private:
     NestingNodeContext(const NestingNodeContext *parent,
-                       Definition *,int index,int level,bool addClasses,bool inherit,bool hideSuper);
+                       const Definition *,int index,int level,bool addClasses,bool inherit,bool hideSuper);
    ~NestingNodeContext();
     class Private;
     Private *p;
@@ -534,8 +537,8 @@ class NestingContext : public RefCountedContext, public TemplateListIntf
     { return new NestingContext(parent,level); }
 
     // TemplateListIntf
-    virtual int  count() const;
-    virtual TemplateVariant at(int index) const;
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
     virtual TemplateListIntf::ConstIterator *createIterator() const;
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
@@ -544,7 +547,7 @@ class NestingContext : public RefCountedContext, public TemplateListIntf
     void addClasses(const ClassSDict &clDict,bool rootOnly);
     void addDirs(const DirSDict &);
     void addDirs(const DirList &);
-    void addFiles(const FileNameList &);
+    void addFiles(const FileNameLinkedMap &);
     void addFiles(const FileList &);
     void addPages(const PageSDict &pages,bool rootOnly);
     void addModules(const GroupSDict &modules);
@@ -586,8 +589,8 @@ class NamespaceListContext : public RefCountedContext, public TemplateListIntf
     static NamespaceListContext *alloc() { return new NamespaceListContext; }
 
     // TemplateListIntf
-    virtual int  count() const;
-    virtual TemplateVariant at(int index) const;
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
     virtual TemplateListIntf::ConstIterator *createIterator() const;
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
@@ -626,8 +629,8 @@ class DirListContext : public RefCountedContext, public TemplateListIntf
     static DirListContext *alloc() { return new DirListContext; }
 
     // TemplateListIntf
-    virtual int  count() const;
-    virtual TemplateVariant at(int index) const;
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
     virtual TemplateListIntf::ConstIterator *createIterator() const;
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
@@ -647,8 +650,8 @@ class FileListContext : public RefCountedContext, public TemplateListIntf
     static FileListContext *alloc() { return new FileListContext; }
 
     // TemplateListIntf
-    virtual int  count() const;
-    virtual TemplateVariant at(int index) const;
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
     virtual TemplateListIntf::ConstIterator *createIterator() const;
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
@@ -687,8 +690,8 @@ class PageListContext : public RefCountedContext, public TemplateListIntf
     static PageListContext *alloc(const PageSDict *pages) { return new PageListContext(pages); }
 
     // TemplateListIntf methods
-    virtual int  count() const;
-    virtual TemplateVariant at(int index) const;
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
     virtual TemplateListIntf::ConstIterator *createIterator() const;
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
@@ -707,7 +710,7 @@ class PageListContext : public RefCountedContext, public TemplateListIntf
 class PageTreeContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static PageTreeContext *alloc() { return new PageTreeContext; }
+    static PageTreeContext *alloc(const PageSDict *pages) { return new PageTreeContext(pages); }
 
     // TemplateStructIntf methods
     virtual TemplateVariant get(const char *name) const;
@@ -715,7 +718,7 @@ class PageTreeContext : public RefCountedContext, public TemplateStructIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    PageTreeContext();
+    PageTreeContext(const PageSDict *pages);
    ~PageTreeContext();
     class Private;
     Private *p;
@@ -726,7 +729,7 @@ class PageTreeContext : public RefCountedContext, public TemplateStructIntf
 class ModuleNodeContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static ModuleNodeContext *alloc(GroupDef *gd) { return new ModuleNodeContext(gd); }
+    static ModuleNodeContext *alloc(const GroupDef *gd) { return new ModuleNodeContext(gd); }
 
     // TemplateStructIntf methods
     virtual TemplateVariant get(const char *name) const;
@@ -734,7 +737,7 @@ class ModuleNodeContext : public RefCountedContext, public TemplateStructIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    ModuleNodeContext(GroupDef *);
+    ModuleNodeContext(const GroupDef *);
    ~ModuleNodeContext();
     class Private;
     Private *p;
@@ -748,8 +751,8 @@ class ModuleListContext : public RefCountedContext, public TemplateListIntf
     static ModuleListContext *alloc() { return new ModuleListContext(); }
 
     // TemplateListIntf
-    virtual int  count() const;
-    virtual TemplateVariant at(int index) const;
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
     virtual TemplateListIntf::ConstIterator *createIterator() const;
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
@@ -785,10 +788,32 @@ class ModuleTreeContext : public RefCountedContext, public TemplateStructIntf
 
 //----------------------------------------------------
 
-class ExampleListContext : public RefCountedContext, public TemplateStructIntf
+class ExampleListContext : public RefCountedContext, public TemplateListIntf
 {
   public:
-    static ExampleListContext *alloc() { return new ExampleListContext(); }
+    static ExampleListContext *alloc() { return new ExampleListContext; }
+
+    // TemplateListIntf methods
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
+    virtual TemplateListIntf::ConstIterator *createIterator() const;
+    virtual int addRef()  { return RefCountedContext::addRef(); }
+    virtual int release() { return RefCountedContext::release(); }
+
+  private:
+    ExampleListContext();
+   ~ExampleListContext();
+    class Private;
+    Private *p;
+};
+
+//----------------------------------------------------
+
+
+class ExampleTreeContext : public RefCountedContext, public TemplateStructIntf
+{
+  public:
+    static ExampleTreeContext *alloc() { return new ExampleTreeContext; }
 
     // TemplateStructIntf methods
     virtual TemplateVariant get(const char *name) const;
@@ -796,8 +821,8 @@ class ExampleListContext : public RefCountedContext, public TemplateStructIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    ExampleListContext();
-   ~ExampleListContext();
+    ExampleTreeContext();
+   ~ExampleTreeContext();
     class Private;
     Private *p;
 };
@@ -864,7 +889,7 @@ class NamespaceMembersIndexContext : public RefCountedContext, public TemplateSt
 class NavPathElemContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static NavPathElemContext *alloc(Definition *def) { return new NavPathElemContext(def); }
+    static NavPathElemContext *alloc(const Definition *def) { return new NavPathElemContext(def); }
 
     // TemplateStructIntf methods
     virtual TemplateVariant get(const char *name) const;
@@ -872,7 +897,7 @@ class NavPathElemContext : public RefCountedContext, public TemplateStructIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    NavPathElemContext(Definition *def);
+    NavPathElemContext(const Definition *def);
    ~NavPathElemContext();
     class Private;
     Private *p;
@@ -884,7 +909,7 @@ class NavPathElemContext : public RefCountedContext, public TemplateStructIntf
 class InheritanceNodeContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static InheritanceNodeContext *alloc(ClassDef *cd,const QCString &name)
+    static InheritanceNodeContext *alloc(const ClassDef *cd,const QCString &name)
     { return new InheritanceNodeContext(cd,name); }
 
     // TemplateStructIntf methods
@@ -893,7 +918,7 @@ class InheritanceNodeContext : public RefCountedContext, public TemplateStructIn
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    InheritanceNodeContext(ClassDef *cd,const QCString &name);
+    InheritanceNodeContext(const ClassDef *cd,const QCString &name);
    ~InheritanceNodeContext();
     class Private;
     Private *p;
@@ -908,8 +933,8 @@ class InheritanceListContext : public RefCountedContext, public TemplateListIntf
     { return new InheritanceListContext(list,baseClasses); }
 
     // TemplateListIntf
-    virtual int  count() const;
-    virtual TemplateVariant at(int index) const;
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
     virtual TemplateListIntf::ConstIterator *createIterator() const;
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
@@ -934,8 +959,8 @@ class MemberListContext : public RefCountedContext, public TemplateListIntf
     { return new MemberListContext(ml,doSort); }
 
     // TemplateListIntf
-    virtual int  count() const;
-    virtual TemplateVariant at(int index) const;
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
     virtual TemplateListIntf::ConstIterator *createIterator() const;
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
@@ -954,7 +979,7 @@ class MemberListContext : public RefCountedContext, public TemplateListIntf
 class MemberGroupInfoContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static MemberGroupInfoContext *alloc(Definition *def,const QCString &relPath,const MemberGroup *mg)
+    static MemberGroupInfoContext *alloc(const Definition *def,const QCString &relPath,const MemberGroup *mg)
     { return new MemberGroupInfoContext(def,relPath,mg); }
 
     // TemplateStructIntf methods
@@ -963,7 +988,7 @@ class MemberGroupInfoContext : public RefCountedContext, public TemplateStructIn
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    MemberGroupInfoContext(Definition *def,const QCString &relPath,const MemberGroup *mg);
+    MemberGroupInfoContext(const Definition *def,const QCString &relPath,const MemberGroup *mg);
    ~MemberGroupInfoContext();
     class Private;
     Private *p;
@@ -976,22 +1001,22 @@ class MemberGroupListContext : public RefCountedContext, public TemplateListIntf
   public:
     static MemberGroupListContext *alloc()
     { return new MemberGroupListContext; }
-    static MemberGroupListContext *alloc(Definition *def,const QCString &relPath,const MemberGroupList *list)
+    static MemberGroupListContext *alloc(const Definition *def,const QCString &relPath,const MemberGroupList *list)
     { return new MemberGroupListContext(def,relPath,list); }
-    static MemberGroupListContext *alloc(Definition *def,const QCString &relPath,const MemberGroupSDict *dict,bool subGrouping)
+    static MemberGroupListContext *alloc(const Definition *def,const QCString &relPath,const MemberGroupSDict *dict,bool subGrouping)
     { return new MemberGroupListContext(def,relPath,dict,subGrouping); }
 
     // TemplateListIntf
-    virtual int  count() const;
-    virtual TemplateVariant at(int index) const;
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
     virtual TemplateListIntf::ConstIterator *createIterator() const;
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
 
   private:
     MemberGroupListContext();
-    MemberGroupListContext(Definition *def,const QCString &relPath,const MemberGroupList *list);
-    MemberGroupListContext(Definition *def,const QCString &relPath,const MemberGroupSDict *mgDict,bool subGrouping);
+    MemberGroupListContext(const Definition *def,const QCString &relPath,const MemberGroupList *list);
+    MemberGroupListContext(const Definition *def,const QCString &relPath,const MemberGroupSDict *mgDict,bool subGrouping);
    ~MemberGroupListContext();
     class Private;
     Private *p;
@@ -1003,7 +1028,7 @@ class MemberGroupListContext : public RefCountedContext, public TemplateListIntf
 class MemberListInfoContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static MemberListInfoContext *alloc(Definition *def,const QCString &relPath,
+    static MemberListInfoContext *alloc(const Definition *def,const QCString &relPath,
                           const MemberList *ml,const QCString &title,
                           const QCString &subtitle=QCString())
     { return new MemberListInfoContext(def,relPath,ml,title,subtitle); }
@@ -1014,7 +1039,7 @@ class MemberListInfoContext : public RefCountedContext, public TemplateStructInt
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    MemberListInfoContext(Definition *def,const QCString &relPath,
+    MemberListInfoContext(const Definition *def,const QCString &relPath,
                           const MemberList *ml,const QCString &title,
                           const QCString &subtitle=QCString());
    ~MemberListInfoContext();
@@ -1046,7 +1071,7 @@ class MemberInfoContext : public RefCountedContext, public TemplateStructIntf
 class InheritedMemberInfoContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static InheritedMemberInfoContext *alloc(ClassDef *cd,MemberList *ml,const QCString &title)
+    static InheritedMemberInfoContext *alloc(const ClassDef *cd,MemberList *ml,const QCString &title)
     { return new InheritedMemberInfoContext(cd,ml,title); }
 
     // TemplateStructIntf methods
@@ -1055,7 +1080,7 @@ class InheritedMemberInfoContext : public RefCountedContext, public TemplateStru
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    InheritedMemberInfoContext(ClassDef *cd,MemberList *ml,const QCString &title);
+    InheritedMemberInfoContext(const ClassDef *cd,MemberList *ml,const QCString &title);
    ~InheritedMemberInfoContext();
     class Private;
     Private *p;
@@ -1067,11 +1092,11 @@ class InheritedMemberInfoListContext : public RefCountedContext, public Template
 {
   public:
     static InheritedMemberInfoListContext *alloc() { return new InheritedMemberInfoListContext; }
-    void addMemberList(ClassDef *cd,MemberListType lt,const QCString &title,bool additionalList=TRUE);
+    void addMemberList(const ClassDef *cd,MemberListType lt,const QCString &title,bool additionalList=TRUE);
 
     // TemplateListIntf
-    virtual int  count() const;
-    virtual TemplateVariant at(int index) const;
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
     virtual TemplateListIntf::ConstIterator *createIterator() const;
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
@@ -1094,8 +1119,8 @@ class AllMembersListContext : public RefCountedContext, public TemplateListIntf
     { return new AllMembersListContext(ml); }
 
     // TemplateListIntf
-    virtual int  count() const;
-    virtual TemplateVariant at(int index) const;
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
     virtual TemplateListIntf::ConstIterator *createIterator() const;
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
@@ -1113,7 +1138,7 @@ class AllMembersListContext : public RefCountedContext, public TemplateListIntf
 class ArgumentContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static ArgumentContext *alloc(const Argument *arg,Definition *def,const QCString &relPath)
+    static ArgumentContext *alloc(const Argument &arg,const Definition *def,const QCString &relPath)
     { return new ArgumentContext(arg,def,relPath); }
 
     // TemplateStructIntf methods
@@ -1122,7 +1147,7 @@ class ArgumentContext : public RefCountedContext, public TemplateStructIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    ArgumentContext(const Argument *arg,Definition *def,const QCString &relPath);
+    ArgumentContext(const Argument &arg,const Definition *def,const QCString &relPath);
    ~ArgumentContext();
     class Private;
     Private *p;
@@ -1134,19 +1159,19 @@ class ArgumentListContext : public RefCountedContext, public TemplateListIntf
 {
   public:
     static ArgumentListContext *alloc() { return new ArgumentListContext; }
-    static ArgumentListContext *alloc(const ArgumentList *al,Definition *def,const QCString &relPath)
+    static ArgumentListContext *alloc(const ArgumentList &al,const Definition *def,const QCString &relPath)
     { return new ArgumentListContext(al,def,relPath); }
 
     // TemplateListIntf
-    virtual int  count() const;
-    virtual TemplateVariant at(int index) const;
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
     virtual TemplateListIntf::ConstIterator *createIterator() const;
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
 
   private:
     ArgumentListContext();
-    ArgumentListContext(const ArgumentList *al,Definition *def,const QCString &relPath);
+    ArgumentListContext(const ArgumentList &al,const Definition *def,const QCString &relPath);
    ~ArgumentListContext();
     class Private;
     Private *p;
@@ -1154,6 +1179,174 @@ class ArgumentListContext : public RefCountedContext, public TemplateListIntf
 
 //----------------------------------------------------
 
+class SymbolContext : public RefCountedContext, public TemplateStructIntf
+{
+  public:
+    static SymbolContext *alloc(const Definition *def,const Definition *prev,const Definition *next)
+    { return new SymbolContext(def,prev,next); }
+
+    // TemplateStructIntf methods
+    virtual TemplateVariant get(const char *name) const;
+    virtual int addRef()  { return RefCountedContext::addRef(); }
+    virtual int release() { return RefCountedContext::release(); }
+
+  private:
+    SymbolContext(const Definition *def,const Definition *prev,const Definition *next);
+   ~SymbolContext();
+    class Private;
+    Private *p;
+};
+
+//----------------------------------------------------
+
+class SymbolListContext : public RefCountedContext, public TemplateListIntf
+{
+  public:
+    static SymbolListContext *alloc(const SearchDefinitionList *sdl)
+    { return new SymbolListContext(sdl); }
+
+    // TemplateListIntf
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
+    virtual TemplateListIntf::ConstIterator *createIterator() const;
+    virtual int addRef()  { return RefCountedContext::addRef(); }
+    virtual int release() { return RefCountedContext::release(); }
+
+  private:
+    SymbolListContext(const SearchDefinitionList *sdl);
+   ~SymbolListContext();
+    class Private;
+    Private *p;
+};
+
+//----------------------------------------------------
+
+class SymbolGroupContext : public RefCountedContext, public TemplateStructIntf
+{
+  public:
+    static SymbolGroupContext *alloc(const SearchDefinitionList *sdl)
+    { return new SymbolGroupContext(sdl); }
+
+    // TemplateStructIntf methods
+    virtual TemplateVariant get(const char *name) const;
+    virtual int addRef()  { return RefCountedContext::addRef(); }
+    virtual int release() { return RefCountedContext::release(); }
+
+  private:
+    SymbolGroupContext(const SearchDefinitionList *sdl);
+   ~SymbolGroupContext();
+    class Private;
+    Private *p;
+};
+
+//----------------------------------------------------
+
+class SymbolGroupListContext : public RefCountedContext, public TemplateListIntf
+{
+  public:
+    static SymbolGroupListContext *alloc(const SearchIndexList *sil)
+    { return new SymbolGroupListContext(sil); }
+
+    // TemplateListIntf
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
+    virtual TemplateListIntf::ConstIterator *createIterator() const;
+    virtual int addRef()  { return RefCountedContext::addRef(); }
+    virtual int release() { return RefCountedContext::release(); }
+
+  private:
+    SymbolGroupListContext(const SearchIndexList *sil);
+   ~SymbolGroupListContext();
+    class Private;
+    Private *p;
+};
+
+//----------------------------------------------------
+
+class SymbolIndexContext : public RefCountedContext, public TemplateStructIntf
+{
+  public:
+    static SymbolIndexContext *alloc(const SearchIndexList *sl,const QCString &name)
+    { return new SymbolIndexContext(sl,name); }
+
+    // TemplateStructIntf methods
+    virtual TemplateVariant get(const char *name) const;
+    virtual int addRef()  { return RefCountedContext::addRef(); }
+    virtual int release() { return RefCountedContext::release(); }
+
+  private:
+    SymbolIndexContext(const SearchIndexList *sl,const QCString &name);
+   ~SymbolIndexContext();
+    class Private;
+    Private *p;
+};
+
+//----------------------------------------------------
+
+class SymbolIndicesContext : public RefCountedContext, public TemplateListIntf
+{
+  public:
+    static SymbolIndicesContext *alloc(const SearchIndexInfo *info)
+    { return new SymbolIndicesContext(info); }
+
+    // TemplateListIntf
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
+    virtual TemplateListIntf::ConstIterator *createIterator() const;
+    virtual int addRef()  { return RefCountedContext::addRef(); }
+    virtual int release() { return RefCountedContext::release(); }
+
+  private:
+    SymbolIndicesContext(const SearchIndexInfo *info);
+   ~SymbolIndicesContext();
+    class Private;
+    Private *p;
+};
+
+//----------------------------------------------------
+
+class SearchIndexContext : public RefCountedContext, public TemplateStructIntf
+{
+  public:
+    static SearchIndexContext *alloc(const SearchIndexInfo *info)
+    { return new SearchIndexContext(info); }
+
+    // TemplateStructIntf methods
+    virtual TemplateVariant get(const char *name) const;
+    virtual int addRef()  { return RefCountedContext::addRef(); }
+    virtual int release() { return RefCountedContext::release(); }
+
+  private:
+    SearchIndexContext(const SearchIndexInfo *info);
+   ~SearchIndexContext();
+    class Private;
+    Private *p;
+};
+
+//----------------------------------------------------
+
+class SearchIndicesContext : public RefCountedContext, public TemplateListIntf
+{
+  public:
+    static SearchIndicesContext *alloc() { return new SearchIndicesContext; }
+
+    // TemplateListIntf
+    virtual uint count() const;
+    virtual TemplateVariant at(uint index) const;
+    virtual TemplateListIntf::ConstIterator *createIterator() const;
+    virtual int addRef()  { return RefCountedContext::addRef(); }
+    virtual int release() { return RefCountedContext::release(); }
+
+  private:
+    SearchIndicesContext();
+   ~SearchIndicesContext();
+    class Private;
+    Private *p;
+};
+
+//----------------------------------------------------
+
 void generateOutputViaTemplate();
+void generateTemplateFiles(const char *templateDir);
 
 #endif
