@@ -1,13 +1,13 @@
 /******************************************************************************
  *
- * 
+ *
  *
  *
  * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
- * documentation under the terms of the GNU General Public License is hereby 
- * granted. No representations are made about the suitability of this software 
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
  * for any purpose. It is provided "as is" without express or implied warranty.
  * See the GNU General Public License for more details.
  *
@@ -71,14 +71,14 @@ void generateDEFForMember(MemberDef *md,
   // + source definition
   // - source references
   // - source referenced by
-  // - include code 
+  // - include code
 
   if (md->memberType()==MemberType_EnumValue) return;
 
   QCString scopeName;
-  if (md->getClassDef()) 
+  if (md->getClassDef())
     scopeName=md->getClassDef()->name();
-  else if (md->getNamespaceDef()) 
+  else if (md->getNamespaceDef())
     scopeName=md->getNamespaceDef()->name();
 
   t << "    " << Prefix << "-member = {" << endl;
@@ -144,71 +144,66 @@ void generateDEFForMember(MemberDef *md,
 
   if (isFunc) //function
   {
-    ArgumentList *declAl = new ArgumentList;
-    const ArgumentList *defAl = md->argumentList();
-    stringToArgumentList(md->argsString(),declAl);
+    const ArgumentList &defAl = md->argumentList();
+    ArgumentList declAl;
+    stringToArgumentList(md->getLanguage(),md->argsString(),declAl);
     QCString fcnPrefix = "  " + memPrefix + "param-";
 
-    if (defAl && declAl->count()>0)
+    auto defIt = defAl.begin();
+    for (const Argument &a : declAl)
     {
-      ArgumentListIterator declAli(*declAl);
-      ArgumentListIterator defAli(*defAl);
-      Argument *a;
-      for (declAli.toFirst();(a=declAli.current());++declAli)
+      const Argument *defArg = 0;
+      if (defIt!=defAl.end())
       {
-        Argument *defArg = defAli.current();
-        t << memPrefix << "param = {" << endl;
-        if (!a->attrib.isEmpty())
-        {
-          t << fcnPrefix << "attributes = ";
-          writeDEFString(t,a->attrib);
-          t << ';' << endl;
-        }
-        if (!a->type.isEmpty())
-        {
-          t << fcnPrefix << "type = <<_EnD_oF_dEf_TeXt_" << endl
-            << a->type << endl << "_EnD_oF_dEf_TeXt_;" << endl;
-        }
-        if (!a->name.isEmpty())
-        {
-          t << fcnPrefix << "declname = ";
-          writeDEFString(t,a->name);
-          t << ';' << endl;
-        }
-        if (defArg && !defArg->name.isEmpty() && defArg->name!=a->name)
-        {
-          t << fcnPrefix << "defname = ";
-          writeDEFString(t,defArg->name);
-          t << ';' << endl;
-        }
-        if (!a->array.isEmpty())
-        {
-          t << fcnPrefix << "array = ";
-          writeDEFString(t,a->array); 
-          t << ';' << endl;
-        }
-        if (!a->defval.isEmpty())
-        {
-          t << fcnPrefix << "defval = <<_EnD_oF_dEf_TeXt_" << endl
-            << a->defval << endl << "_EnD_oF_dEf_TeXt_;" << endl;
-        }
-        if (defArg) ++defAli;
-        t << "      }; /*" << fcnPrefix << "-param */" << endl;
+        defArg = &(*defIt);
+        ++defIt;
       }
+      t << memPrefix << "param = {" << endl;
+      if (!a.attrib.isEmpty())
+      {
+        t << fcnPrefix << "attributes = ";
+        writeDEFString(t,a.attrib);
+        t << ';' << endl;
+      }
+      if (!a.type.isEmpty())
+      {
+        t << fcnPrefix << "type = <<_EnD_oF_dEf_TeXt_" << endl
+          << a.type << endl << "_EnD_oF_dEf_TeXt_;" << endl;
+      }
+      if (!a.name.isEmpty())
+      {
+        t << fcnPrefix << "declname = ";
+        writeDEFString(t,a.name);
+        t << ';' << endl;
+      }
+      if (defArg && !defArg->name.isEmpty() && defArg->name!=a.name)
+      {
+        t << fcnPrefix << "defname = ";
+        writeDEFString(t,defArg->name);
+        t << ';' << endl;
+      }
+      if (!a.array.isEmpty())
+      {
+        t << fcnPrefix << "array = ";
+        writeDEFString(t,a.array);
+        t << ';' << endl;
+      }
+      if (!a.defval.isEmpty())
+      {
+        t << fcnPrefix << "defval = <<_EnD_oF_dEf_TeXt_" << endl
+          << a.defval << endl << "_EnD_oF_dEf_TeXt_;" << endl;
+      }
+      t << "      }; /*" << fcnPrefix << "-param */" << endl;
     }
-    delete declAl;
   }
   else if (  md->memberType()==MemberType_Define
       && md->argsString()!=0)
   {
-    ArgumentListIterator ali(*md->argumentList());
-    Argument *a;
     QCString defPrefix = "  " + memPrefix + "def-";
-
-    for (ali.toFirst();(a=ali.current());++ali)
+    for (const Argument &a : md->argumentList())
     {
       t << memPrefix << "param  = {" << endl;
-      t << defPrefix << "name = '" << a->type << "';" << endl;
+      t << defPrefix << "name = '" << a.type << "';" << endl;
       t << "      }; /*" << defPrefix << "-param */" << endl;
     }
   }
@@ -293,7 +288,7 @@ void generateDEFForMember(MemberDef *md,
     {
       if (rmd->getStartBodyLine()!=-1 && rmd->getBodyDef())
       {
-        t << memPrefix << "referenceby = {" << endl;
+        t << memPrefix << "referencedby = {" << endl;
         t << refPrefix << "id = '"
           << rmd->getBodyDef()->getOutputFileBase()
           << "_1"   // encoded ':' character (see util.cpp:convertNameToFile)
@@ -577,9 +572,8 @@ void generateDEF()
       dir.setPath(QDir::currentDirPath());
       if (!dir.mkdir(outputDirectory))
       {
-        err("tag OUTPUT_DIRECTORY: Output directory '%s' does not "
+        term("tag OUTPUT_DIRECTORY: Output directory '%s' does not "
             "exist and cannot be created\n",outputDirectory.data());
-        exit(1);
       }
       else
       {
@@ -618,7 +612,7 @@ void generateDEF()
   FTextStream t(&f);
   t << "AutoGen Definitions dummy;" << endl;
 
-  if (Doxygen::classSDict->count()+Doxygen::inputNameList->count()>0)
+  if (Doxygen::classSDict->count()+Doxygen::inputNameLinkedMap->size()>0)
   {
     ClassSDict::Iterator cli(*Doxygen::classSDict);
     ClassDef *cd;
@@ -626,15 +620,11 @@ void generateDEF()
     {
       generateDEFForClass(cd,t);
     }
-    FileNameListIterator fnli(*Doxygen::inputNameList);
-    FileName *fn;
-    for (;(fn=fnli.current());++fnli)
+    for (const auto &fn : *Doxygen::inputNameLinkedMap)
     {
-      FileNameIterator fni(*fn);
-      FileDef *fd;
-      for (;(fd=fni.current());++fni)
+      for (const auto &fd : *fn)
       {
-        generateDEFForFile(fd,t);
+        generateDEFForFile(fd.get(),t);
       }
     }
   }
