@@ -18,6 +18,7 @@
 #ifndef DEFINITION_H
 #define DEFINITION_H
 
+#include <vector>
 #include <qlist.h>
 #include <qdict.h>
 
@@ -31,15 +32,15 @@
 
 class FileDef;
 class OutputList;
-class SectionDict;
+class SectionRefs;
 class MemberSDict;
 class MemberDef;
 class GroupDef;
 class GroupList;
-struct ListItemInfo;
-struct SectionInfo;
+class SectionInfo;
 class Definition;
 class FTextStream;
+class RefItem;
   
 /** Data associated with a detailed description. */
 struct DocInfo
@@ -61,8 +62,9 @@ struct BriefInfo
 /** Data associated with description found in the body. */
 struct BodyInfo
 {
-    int      startLine;   //!< line number of the start of the definition
-    int      endLine;     //!< line number of the end of the definition
+    int      defLine;     //!< line number of the start of the definition
+    int      startLine;   //!< line number of the start of the definition's body
+    int      endLine;     //!< line number of the end of the definition's body
     FileDef *fileDef;     //!< file definition containing the function body
 };
     
@@ -111,6 +113,11 @@ class Definition : public DefinitionIntf
     /*! Returns the name of the definition */
     virtual QCString name() const = 0;
 
+    /*! Returns TRUE iff this definition has an artificially generated name
+     * (typically starting with a @) that is used for nameless definitions
+     */
+    virtual bool isAnonymous() const = 0;
+
     /*! Returns the name of the definition as it appears in the output */
     virtual QCString displayName(bool includeScope=TRUE) const = 0;
 
@@ -182,7 +189,7 @@ class Definition : public DefinitionIntf
     /*! returns the extension of the file in which this definition was found */
     virtual QCString getDefFileExtension() const = 0;
 
-    /*! returns the line number at which the definition was found */
+    /*! returns the line number at which the definition was found (can be the declaration) */
     virtual int getDefLine() const = 0;
 
     /*! returns the column number at which the definition was found */
@@ -236,6 +243,9 @@ class Definition : public DefinitionIntf
     /*! Convenience method to return a resolved external link */
     virtual QCString externalReference(const QCString &relPath) const = 0;
 
+    /*! Returns the first line of the implementation of this item. See also getDefLine() */
+    virtual int getStartDefLine() const = 0;
+
     /*! Returns the first line of the body of this item (applicable to classes and 
      *  functions).
      */
@@ -257,7 +267,7 @@ class Definition : public DefinitionIntf
     virtual GroupList *partOfGroups() const = 0;
     virtual bool isLinkableViaGroup() const = 0;
 
-    virtual QList<ListItemInfo> *xrefListItems() const = 0;
+    virtual const std::vector<RefItem*> &xrefListItems() const = 0;
 
     virtual Definition *findInnerCompound(const char *name) const = 0;
     virtual Definition *getOuterScope() const = 0;
@@ -274,7 +284,7 @@ class Definition : public DefinitionIntf
     virtual QCString id() const = 0;
 
     /** returns the section dictionary, only of importance for pagedef */
-    virtual SectionDict * getSectionDict() const = 0;
+    virtual const SectionRefs &getSectionRefs() const = 0;
 
     virtual QCString navigationPathAsString() const = 0;
     virtual QCString pathFragment() const = 0;
@@ -310,10 +320,10 @@ class Definition : public DefinitionIntf
     virtual void setReference(const char *r) = 0;
 
     // source references
-    virtual void setBodySegment(int bls,int ble) = 0;
+    virtual void setBodySegment(int defLine, int bls,int ble) = 0;
     virtual void setBodyDef(FileDef *fd) = 0;
 
-    virtual void setRefItems(const QList<ListItemInfo> *sli) = 0;
+    virtual void setRefItems(const std::vector<RefItem*> &sli) = 0;
     virtual void setOuterScope(Definition *d) = 0;
 
     virtual void setHidden(bool b) = 0;
@@ -331,7 +341,7 @@ class Definition : public DefinitionIntf
     /*! Add the list of anchors that mark the sections that are found in the 
      * documentation.
      */
-    virtual void addSectionsToDefinition(QList<SectionInfo> *anchorList) = 0;
+    virtual void addSectionsToDefinition(const std::vector<const SectionInfo*> &anchorList) = 0;
     virtual void addSourceReferencedBy(const MemberDef *d) = 0;
     virtual void addSourceReferences(const MemberDef *d) = 0;
     virtual void mergeRefItems(Definition *d) = 0;
