@@ -8,20 +8,94 @@
 #include "language.h"
 #include "message.h"
 #include "dot.h"
+#include "dotdirdeps.h"
 #include "layout.h"
 #include "ftextstream.h"
 #include "config.h"
 #include "docparser.h"
+#include "definitionimpl.h"
+#include "filedef.h"
+
+//----------------------------------------------------------------------
+
+class DirDefImpl : public DefinitionImpl, public DirDef
+{
+  public:
+    DirDefImpl(const char *path);
+    virtual ~DirDefImpl();
+
+    virtual DefType definitionType() const { return TypeDir; }
+    virtual QCString getOutputFileBase() const;
+    virtual QCString anchor() const { return QCString(); }
+    virtual bool isLinkableInProject() const;
+    virtual bool isLinkable() const;
+    virtual QCString displayName(bool=TRUE) const { return m_dispName; }
+    virtual const QCString &shortName() const { return m_shortName; }
+    virtual void addSubDir(DirDef *subdir);
+    virtual FileList *   getFiles() const        { return m_fileList; }
+    virtual void addFile(FileDef *fd);
+    virtual const DirList &subDirs() const { return m_subdirs; }
+    virtual bool isCluster() const { return m_subdirs.count()>0; }
+    virtual int level() const { return m_level; }
+    virtual DirDef *parent() const { return m_parent; }
+    virtual int dirCount() const { return m_dirCount; }
+    virtual const QDict<UsedDir> *usedDirs() const { return m_usedDirs; }
+    virtual bool isParentOf(const DirDef *dir) const;
+    virtual bool depGraphIsTrivial() const;
+    virtual QCString shortTitle() const;
+    virtual bool hasDetailedDescription() const;
+    virtual void writeDocumentation(OutputList &ol);
+    virtual void writeTagFile(FTextStream &t);
+    virtual void setDiskName(const QCString &name) { m_diskName = name; }
+    virtual void sort();
+    virtual void setParent(DirDef *parent);
+    virtual void setLevel();
+    virtual void addUsesDependency(DirDef *usedDir,FileDef *srcFd,
+                           FileDef *dstFd,bool inherited);
+    virtual void computeDependencies();
+
+  public:
+    static DirDef *mergeDirectoryInTree(const QCString &path);
+
+  private:
+
+    void writeDetailedDescription(OutputList &ol,const QCString &title);
+    void writeBriefDescription(OutputList &ol);
+    void writeDirectoryGraph(OutputList &ol);
+    void writeSubDirList(OutputList &ol);
+    void writeFileList(OutputList &ol);
+    void startMemberDeclarations(OutputList &ol);
+    void endMemberDeclarations(OutputList &ol);
+
+    static DirDef *createNewDir(const char *path);
+    static bool matchPath(const QCString &path,QStrList &l);
+
+    DirList m_subdirs;
+    QCString m_dispName;
+    QCString m_shortName;
+    QCString m_diskName;
+    FileList *m_fileList;                 // list of files in the group
+    int m_dirCount;
+    int m_level;
+    DirDef *m_parent;
+    QDict<UsedDir> *m_usedDirs;
+};
+
+DirDef *createDirDef(const char *path)
+{
+  return new DirDefImpl(path);
+}
+
 
 //----------------------------------------------------------------------
 // method implementation
 
 static int g_dirCount=0;
 
-DirDef::DirDef(const char *path) : Definition(path,1,1,path), visited(FALSE)
+DirDefImpl::DirDefImpl(const char *path) : DefinitionImpl(path,1,1,path)
 {
   bool fullPathNames = Config_getBool(FULL_PATH_NAMES);
-  // get display name (stipping the paths mentioned in STRIP_FROM_PATH)
+  // get display name (stripping the paths mentioned in STRIP_FROM_PATH)
   // get short name (last part of path)
   m_shortName = path;
   m_diskName = path;
@@ -30,7 +104,7 @@ DirDef::DirDef(const char *path) : Definition(path,1,1,path), visited(FALSE)
     m_shortName = m_shortName.left(m_shortName.length()-1);
   }
   int pi=m_shortName.findRev('/');
-  if (pi!=-1) 
+  if (pi!=-1)
   { // remove everything till the last /
     m_shortName = m_shortName.mid(pi+1);
   }
@@ -40,7 +114,7 @@ DirDef::DirDef(const char *path) : Definition(path,1,1,path), visited(FALSE)
   { // strip trailing /
     m_dispName = m_dispName.left(m_dispName.length()-1);
   }
-  
+
   m_fileList   = new FileList;
   m_usedDirs   = new QDict<UsedDir>(257);
   m_usedDirs->setAutoDelete(TRUE);
@@ -49,36 +123,41 @@ DirDef::DirDef(const char *path) : Definition(path,1,1,path), visited(FALSE)
   m_parent=0;
 }
 
-DirDef::~DirDef()
+DirDefImpl::~DirDefImpl()
 {
   delete m_fileList;
   delete m_usedDirs;
 }
 
-bool DirDef::isLinkableInProject() const 
-{ 
-  return !isReference(); 
+bool DirDefImpl::isLinkableInProject() const
+{
+  return !isReference();
 }
 
-bool DirDef::isLinkable() const 
-{ 
-  return isReference() || isLinkableInProject(); 
+bool DirDefImpl::isLinkable() const
+{
+  return isReference() || isLinkableInProject();
 }
 
-void DirDef::addSubDir(DirDef *subdir)
+void DirDefImpl::addSubDir(DirDef *subdir)
 {
   m_subdirs.append(subdir);
   subdir->setOuterScope(this);
-  subdir->m_parent=this;
+  subdir->setParent(this);
 }
 
-void DirDef::addFile(FileDef *fd)
+void DirDefImpl::setParent(DirDef *p)
+{
+   m_parent=p;
+}
+
+void DirDefImpl::addFile(FileDef *fd)
 {
   m_fileList->append(fd);
   fd->setDirDef(this);
 }
 
-void DirDef::sort()
+void DirDefImpl::sort()
 {
   m_subdirs.sort();
   m_fileList->sort();
@@ -116,16 +195,16 @@ static QCString encodeDirName(const QCString &anchor)
 //  return result;
 }
 
-QCString DirDef::getOutputFileBase() const
+QCString DirDefImpl::getOutputFileBase() const
 {
-  //printf("DirDef::getOutputFileBase() %s->dir_%s\n",
+  //printf("DirDefImpl::getOutputFileBase() %s->dir_%s\n",
   //    m_diskName.data(),encodeDirName(m_diskName).data());
   return "dir_"+encodeDirName(m_diskName);
 }
 
-void DirDef::writeDetailedDescription(OutputList &ol,const QCString &title)
+void DirDefImpl::writeDetailedDescription(OutputList &ol,const QCString &title)
 {
-  if ((!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF)) || 
+  if ((!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF)) ||
       !documentation().isEmpty())
   {
     ol.pushGeneratorState();
@@ -146,7 +225,7 @@ void DirDef::writeDetailedDescription(OutputList &ol,const QCString &title)
       ol.generateDoc(briefFile(),briefLine(),this,0,briefDescription(),FALSE,FALSE);
     }
     // separator between brief and details
-    if (!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF) && 
+    if (!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF) &&
         !documentation().isEmpty())
     {
       ol.pushGeneratorState();
@@ -168,7 +247,7 @@ void DirDef::writeDetailedDescription(OutputList &ol,const QCString &title)
   }
 }
 
-void DirDef::writeBriefDescription(OutputList &ol)
+void DirDefImpl::writeBriefDescription(OutputList &ol)
 {
   if (hasBriefDescription())
   {
@@ -205,7 +284,7 @@ void DirDef::writeBriefDescription(OutputList &ol)
   ol.writeSynopsis();
 }
 
-void DirDef::writeDirectoryGraph(OutputList &ol)
+void DirDefImpl::writeDirectoryGraph(OutputList &ol)
 {
   // write graph dependency graph
   if (Config_getBool(DIRECTORY_GRAPH) && Config_getBool(HAVE_DOT))
@@ -225,7 +304,7 @@ void DirDef::writeDirectoryGraph(OutputList &ol)
   }
 }
 
-void DirDef::writeSubDirList(OutputList &ol)
+void DirDefImpl::writeSubDirList(OutputList &ol)
 {
   int numSubdirs = 0;
   QListIterator<DirDef> it(m_subdirs);
@@ -275,7 +354,7 @@ void DirDef::writeSubDirList(OutputList &ol)
   }
 }
 
-void DirDef::writeFileList(OutputList &ol)
+void DirDefImpl::writeFileList(OutputList &ol)
 {
   int numFiles = 0;
   QListIterator<FileDef> it(*m_fileList);
@@ -295,9 +374,7 @@ void DirDef::writeFileList(OutputList &ol)
     ol.parseText(theTranslator->trFile(TRUE,FALSE));
     ol.endMemberHeader();
     ol.startMemberList();
-    QListIterator<FileDef> it(*m_fileList);
-    FileDef *fd;
-    for (;(fd=it.current());++it)
+    for (it.toFirst();(fd=it.current());++it)
     {
       if (fd->hasDocumentation())
       {
@@ -347,28 +424,28 @@ void DirDef::writeFileList(OutputList &ol)
   }
 }
 
-void DirDef::startMemberDeclarations(OutputList &ol)
+void DirDefImpl::startMemberDeclarations(OutputList &ol)
 {
   ol.startMemberSections();
 }
 
-void DirDef::endMemberDeclarations(OutputList &ol)
+void DirDefImpl::endMemberDeclarations(OutputList &ol)
 {
   ol.endMemberSections();
 }
 
-QCString DirDef::shortTitle() const
+QCString DirDefImpl::shortTitle() const
 {
   return theTranslator->trDirReference(m_shortName);
 }
 
-bool DirDef::hasDetailedDescription() const
+bool DirDefImpl::hasDetailedDescription() const
 {
   static bool repeatBrief = Config_getBool(REPEAT_BRIEF);
   return (!briefDescription().isEmpty() && repeatBrief) || !documentation().isEmpty();
 }
 
-void DirDef::writeTagFile(FTextStream &tagFile)
+void DirDefImpl::writeTagFile(FTextStream &tagFile)
 {
   tagFile << "  <compound kind=\"dir\">" << endl;
   tagFile << "    <name>" << convertToXML(displayName()) << "</name>" << endl;
@@ -415,11 +492,11 @@ void DirDef::writeTagFile(FTextStream &tagFile)
   tagFile << "  </compound>" << endl;
 }
 
-void DirDef::writeDocumentation(OutputList &ol)
+void DirDefImpl::writeDocumentation(OutputList &ol)
 {
   static bool generateTreeView = Config_getBool(GENERATE_TREEVIEW);
   ol.pushGeneratorState();
-  
+
   QCString title=theTranslator->trDirReference(m_dispName);
   startFile(ol,getOutputFileBase(),name(),title,HLI_Files,!generateTreeView);
 
@@ -451,25 +528,25 @@ void DirDef::writeDocumentation(OutputList &ol)
   {
     switch (lde->kind())
     {
-      case LayoutDocEntry::BriefDesc: 
+      case LayoutDocEntry::BriefDesc:
         writeBriefDescription(ol);
-        break; 
-      case LayoutDocEntry::DirGraph: 
+        break;
+      case LayoutDocEntry::DirGraph:
         writeDirectoryGraph(ol);
-        break; 
-      case LayoutDocEntry::MemberDeclStart: 
+        break;
+      case LayoutDocEntry::MemberDeclStart:
         startMemberDeclarations(ol);
-        break; 
-      case LayoutDocEntry::DirSubDirs: 
+        break;
+      case LayoutDocEntry::DirSubDirs:
         writeSubDirList(ol);
-        break; 
-      case LayoutDocEntry::DirFiles: 
+        break;
+      case LayoutDocEntry::DirFiles:
         writeFileList(ol);
-        break; 
-      case LayoutDocEntry::MemberDeclEnd: 
+        break;
+      case LayoutDocEntry::MemberDeclEnd:
         endMemberDeclarations(ol);
         break;
-      case LayoutDocEntry::DetailedDesc: 
+      case LayoutDocEntry::DetailedDesc:
         {
           LayoutDocEntrySection *ls = (LayoutDocEntrySection*)lde;
           writeDetailedDescription(ol,ls->title(lang));
@@ -497,16 +574,16 @@ void DirDef::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::FileConstantGroups:
       case LayoutDocEntry::FileIncludes:
       case LayoutDocEntry::FileIncludeGraph:
-      case LayoutDocEntry::FileIncludedByGraph: 
+      case LayoutDocEntry::FileIncludedByGraph:
       case LayoutDocEntry::FileSourceLink:
       case LayoutDocEntry::FileInlineClasses:
-      case LayoutDocEntry::GroupClasses: 
-      case LayoutDocEntry::GroupInlineClasses: 
+      case LayoutDocEntry::GroupClasses:
+      case LayoutDocEntry::GroupInlineClasses:
       case LayoutDocEntry::GroupNamespaces:
-      case LayoutDocEntry::GroupDirs: 
-      case LayoutDocEntry::GroupNestedGroups: 
+      case LayoutDocEntry::GroupDirs:
+      case LayoutDocEntry::GroupNestedGroups:
       case LayoutDocEntry::GroupFiles:
-      case LayoutDocEntry::GroupGraph: 
+      case LayoutDocEntry::GroupGraph:
       case LayoutDocEntry::GroupPageDocs:
       case LayoutDocEntry::AuthorSection:
       case LayoutDocEntry::MemberGroups:
@@ -529,7 +606,7 @@ void DirDef::writeDocumentation(OutputList &ol)
   ol.popGeneratorState();
 }
 
-void DirDef::setLevel()
+void DirDefImpl::setLevel()
 {
   if (m_level==-1) // level not set before
   {
@@ -548,8 +625,8 @@ void DirDef::setLevel()
 
 /** Add as "uses" dependency between \a this dir and \a dir,
  *  that was caused by a dependency on file \a fd.
- */ 
-void DirDef::addUsesDependency(DirDef *dir,FileDef *srcFd,
+ */
+void DirDefImpl::addUsesDependency(DirDef *dir,FileDef *srcFd,
                                FileDef *dstFd,bool inherited)
 {
   if (this==dir) return; // do not add self-dependencies
@@ -570,7 +647,7 @@ void DirDef::addUsesDependency(DirDef *dir,FileDef *srcFd,
      if (usedPair==0) // new file dependency
      {
        //printf("  => new file\n");
-       usedDir->addFileDep(srcFd,dstFd); 
+       usedDir->addFileDep(srcFd,dstFd);
        added=TRUE;
      }
      else
@@ -582,7 +659,7 @@ void DirDef::addUsesDependency(DirDef *dir,FileDef *srcFd,
   {
     //printf("  => new file\n");
     usedDir = new UsedDir(dir,inherited);
-    usedDir->addFileDep(srcFd,dstFd); 
+    usedDir->addFileDep(srcFd,dstFd);
     m_usedDirs->insert(dir->getOutputFileBase(),usedDir);
     added=TRUE;
   }
@@ -603,10 +680,10 @@ void DirDef::addUsesDependency(DirDef *dir,FileDef *srcFd,
 
 /** Computes the dependencies between directories
  */
-void DirDef::computeDependencies()
+void DirDefImpl::computeDependencies()
 {
   FileList *fl = m_fileList;
-  if (fl) 
+  if (fl)
   {
     QListIterator<FileDef> fli(*fl);
     FileDef *fd;
@@ -617,7 +694,7 @@ void DirDef::computeDependencies()
       QList<IncludeInfo> *ifl = fd->includeFileList();
       if (ifl)
       {
-        QListIterator<IncludeInfo> ifli(*ifl); 
+        QListIterator<IncludeInfo> ifli(*ifl);
         IncludeInfo *ii;
         for (ifli.toFirst();(ii=ifli.current());++ifli) // foreach include file
         {
@@ -649,17 +726,17 @@ void DirDef::computeDependencies()
   }
 }
 
-bool DirDef::isParentOf(DirDef *dir) const
+bool DirDefImpl::isParentOf(const DirDef *dir) const
 {
-  if (dir->parent()==this) // this is a parent of dir 
+  if (dir->parent()==this) // this is a parent of dir
     return TRUE;
   else if (dir->parent()) // repeat for the parent of dir
-    return isParentOf(dir->parent()); 
+    return isParentOf(dir->parent());
   else
     return FALSE;
 }
 
-bool DirDef::depGraphIsTrivial() const
+bool DirDefImpl::depGraphIsTrivial() const
 {
   return m_usedDirs->count()==0;
 }
@@ -704,21 +781,21 @@ FilePair *UsedDir::findFilePair(const char *name)
   return n.isEmpty() ? 0 : m_filePairs.find(n);
 }
 
-DirDef *DirDef::createNewDir(const char *path)
+DirDef *DirDefImpl::createNewDir(const char *path)
 {
   ASSERT(path!=0);
   DirDef *dir = Doxygen::directories->find(path);
   if (dir==0) // new dir
   {
     //printf("Adding new dir %s\n",path);
-    dir = new DirDef(path);
+    dir = createDirDef(path);
     //printf("createNewDir %s short=%s\n",path,dir->shortName().data());
     Doxygen::directories->append(path,dir);
   }
   return dir;
 }
 
-bool DirDef::matchPath(const QCString &path,QStrList &l)
+bool DirDefImpl::matchPath(const QCString &path,QStrList &l)
 {
   const char *s=l.first();
   while (s)
@@ -736,9 +813,9 @@ bool DirDef::matchPath(const QCString &path,QStrList &l)
 /*! strip part of \a path if it matches
  *  one of the paths in the Config_getList(STRIP_FROM_PATH) list
  */
-DirDef *DirDef::mergeDirectoryInTree(const QCString &path)
+DirDef *DirDefImpl::mergeDirectoryInTree(const QCString &path)
 {
-  //printf("DirDef::mergeDirectoryInTree(%s)\n",path.data());
+  //printf("DirDefImpl::mergeDirectoryInTree(%s)\n",path.data());
   int p=0,i=0;
   DirDef *dir=0;
   while ((i=path.find('/',p))!=-1)
@@ -746,7 +823,7 @@ DirDef *DirDef::mergeDirectoryInTree(const QCString &path)
     QCString part=path.left(i+1);
     if (!matchPath(part,Config_getList(STRIP_FROM_PATH)) && (part!="/" && part!="//"))
     {
-      dir=createNewDir(part); 
+      dir=createNewDir(part);
     }
     p=i+1;
   }
@@ -757,7 +834,7 @@ DirDef *DirDef::mergeDirectoryInTree(const QCString &path)
 
 static void writePartialDirPath(OutputList &ol,const DirDef *root,const DirDef *target)
 {
-  if (target->parent()!=root) 
+  if (target->parent()!=root)
   {
     writePartialDirPath(ol,root,target->parent());
     ol.writeString("&#160;/&#160;");
@@ -834,7 +911,7 @@ void DirRelation::writeDocumentation(OutputList &ol)
   ol.writeString("</table>");
 
   ol.endContents();
-  
+
   endFileWithNavPath(m_src,ol);
 
   ol.popGeneratorState();
@@ -857,10 +934,10 @@ static void computeCommonDirPrefix()
     sdi.toFirst();
     dir=sdi.current();
     path=dir->name();
-    int i=path.findRev('/',path.length()-2);
+    int i=path.findRev('/',(int)path.length()-2);
     path=path.left(i+1);
     bool done=FALSE;
-    if (i==-1) 
+    if (i==-1)
     {
       path="";
     }
@@ -868,8 +945,8 @@ static void computeCommonDirPrefix()
     {
       while (!done)
       {
-        int l = path.length();
-        int count=0;
+        uint l = path.length();
+        uint count=0;
         for (sdi.toFirst();(dir=sdi.current());++sdi)
         {
           QCString dirName = dir->name();
@@ -877,7 +954,7 @@ static void computeCommonDirPrefix()
           {
             if (qstrncmp(dirName,path,l)!=0) // dirName does not start with path
             {
-              int i=path.findRev('/',l-2);
+              i=path.findRev('/',(int)l-2);
               if (i==-1) // no unique prefix -> stop
               {
                 path="";
@@ -894,7 +971,7 @@ static void computeCommonDirPrefix()
           {
             path=dir->name();
             l=path.length();
-            int i=path.findRev('/',l-2);
+            i=path.findRev('/',(int)l-2);
             if (i==-1) // no unique prefix -> stop
             {
               path="";
@@ -927,13 +1004,9 @@ static void computeCommonDirPrefix()
 void buildDirectories()
 {
   // for each input file
-  FileNameListIterator fnli(*Doxygen::inputNameList); 
-  FileName *fn;
-  for (fnli.toFirst();(fn=fnli.current());++fnli)
+  for (const auto &fn : *Doxygen::inputNameLinkedMap)
   {
-    FileNameIterator fni(*fn);
-    FileDef *fd;
-    for (;(fd=fni.current());++fni)
+    for (const auto &fd : *fn)
     {
       //printf("buildDirectories %s\n",fd->name().data());
       if (fd->getReference().isEmpty())
@@ -941,9 +1014,9 @@ void buildDirectories()
         DirDef *dir;
         if ((dir=Doxygen::directories->find(fd->getPath()))==0) // new directory
         {
-          dir = DirDef::mergeDirectoryInTree(fd->getPath());
+          dir = DirDefImpl::mergeDirectoryInTree(fd->getPath());
         }
-        if (dir && !fd->isDocumentationFile()) dir->addFile(fd);
+        if (dir && !fd->isDocumentationFile()) dir->addFile(fd.get());
       }
       else
       {
@@ -959,15 +1032,15 @@ void buildDirectories()
   for (sdi.toFirst();(dir=sdi.current());++sdi)
   {
     QCString name = dir->name();
-    int i=name.findRev('/',name.length()-2);
+    int i=name.findRev('/',(int)name.length()-2);
     if (i>0)
     {
       DirDef *parent = Doxygen::directories->find(name.left(i+1));
       //if (parent==0) parent=root;
-      if (parent) 
+      if (parent)
       {
-        parent->addSubDir(dir); 
-        //printf("DirDef::addSubdir(): Adding subdir\n%s to\n%s\n",
+        parent->addSubDir(dir);
+        //printf("DirDefImpl::addSubdir(): Adding subdir\n%s to\n%s\n",
         //  dir->displayName().data(), parent->displayName().data());
       }
     }
