@@ -33,6 +33,11 @@
 #include "defargs.h"
 #include "outputgen.h"
 #include "dot.h"
+#include "dotcallgraph.h"
+#include "dotclassgraph.h"
+#include "dotdirdeps.h"
+#include "dotgroupcollaboration.h"
+#include "dotincldepgraph.h"
 #include "pagedef.h"
 #include "filename.h"
 #include "version.h"
@@ -127,15 +132,13 @@ void writeDocbookLink(FTextStream &t,const char * /*extRef*/,const char *compoun
   t << "</link>";
 }
 
-DocbookCodeGenerator::DocbookCodeGenerator(FTextStream &t) : m_lineNumber(-1), m_col(0),
-  m_insideCodeLine(FALSE), m_insideSpecialHL(FALSE)
+DocbookCodeGenerator::DocbookCodeGenerator(FTextStream &t)
 {
   m_prettyCode=Config_getBool(DOCBOOK_PROGRAMLISTING);
   setTextStream(t);
 }
 
-DocbookCodeGenerator::DocbookCodeGenerator() : m_lineNumber(-1), m_col(0),
-  m_insideCodeLine(FALSE), m_insideSpecialHL(FALSE),  m_streamSet(FALSE)
+DocbookCodeGenerator::DocbookCodeGenerator()
 {
   m_prettyCode=Config_getBool(DOCBOOK_PROGRAMLISTING);
 }
@@ -153,18 +156,18 @@ void DocbookCodeGenerator::writeCodeLink(const char *ref,const char *file,
 {
   Docbook_DB(("(writeCodeLink)\n"));
   writeDocbookLink(m_t,ref,file,anchor,name,tooltip);
-  m_col+=strlen(name);
+  m_col+=(int)strlen(name);
 }
-void DocbookCodeGenerator::writeCodeLinkLine(const char *ref,const char *file,
-    const char *anchor,const char *name,
-    const char *tooltip)
+void DocbookCodeGenerator::writeCodeLinkLine(const char *,const char *file,
+    const char *,const char *name,
+    const char *)
 {
   Docbook_DB(("(writeCodeLinkLine)\n"));
   m_t << "<anchor xml:id=\"_" << stripExtensionGeneral(stripPath(file),".xml");
   m_t << "_1l";
   writeDocbookString(m_t,name);
   m_t << "\"/>";
-  m_col+=strlen(name);
+  m_col+=(int)strlen(name);
 }
 void DocbookCodeGenerator::writeTooltip(const char *, const DocLinkInfo &, const char *,
                   const char *, const SourceLinkInfo &, const SourceLinkInfo &
@@ -180,7 +183,7 @@ void DocbookCodeGenerator::startCodeLine(bool)
 }
 void DocbookCodeGenerator::endCodeLine()
 {
-  m_t << endl;
+  if (m_insideCodeLine) m_t << endl;
   Docbook_DB(("(endCodeLine)\n"));
   m_lineNumber = -1;
   m_refId.resize(0);
@@ -228,9 +231,9 @@ void DocbookCodeGenerator::writeLineNumber(const char *ref,const char *fileName,
   {
     m_t << l << " ";
   }
-
+  m_col=0;
 }
-void DocbookCodeGenerator::setCurrentDoc(Definition *,const char *,bool)
+void DocbookCodeGenerator::setCurrentDoc(const Definition *,const char *,bool)
 {
 }
 void DocbookCodeGenerator::addWord(const char *,bool)
@@ -238,7 +241,7 @@ void DocbookCodeGenerator::addWord(const char *,bool)
 }
 void DocbookCodeGenerator::finish()
 {
-  if (m_insideCodeLine) endCodeLine();
+  endCodeLine();
 }
 void DocbookCodeGenerator::startCodeFragment()
 {
@@ -246,13 +249,16 @@ void DocbookCodeGenerator::startCodeFragment()
 }
 void DocbookCodeGenerator::endCodeFragment()
 {
+  //endCodeLine checks is there is still an open code line, if so closes it.
+  endCodeLine();
+
   m_t << "</computeroutput></literallayout>" << endl;
 }
 
 DocbookGenerator::DocbookGenerator() : OutputGenerator()
 {
 DB_GEN_C
-  dir=Config_getString(DOCBOOK_OUTPUT);
+  m_dir=Config_getString(DOCBOOK_OUTPUT);
   //insideTabbing=FALSE;
   //firstDescItem=TRUE;
   //disableLinks=FALSE;
@@ -266,8 +272,8 @@ DB_GEN_C
   m_descTable = FALSE;
   m_inLevel = -1;
   m_firstMember = FALSE;
-  for (int i = 0 ; i < sizeof(m_inListItem) / sizeof(*m_inListItem) ; i++) m_inListItem[i] = FALSE;
-  for (int i = 0 ; i < sizeof(m_inSimpleSect) / sizeof(*m_inSimpleSect) ; i++) m_inSimpleSect[i] = FALSE;
+  for (size_t i = 0 ; i < sizeof(m_inListItem) / sizeof(*m_inListItem) ; i++) m_inListItem[i] = FALSE;
+  for (size_t i = 0 ; i < sizeof(m_inSimpleSect) / sizeof(*m_inSimpleSect) ; i++) m_inSimpleSect[i] = FALSE;
 }
 
 DocbookGenerator::~DocbookGenerator()
@@ -281,8 +287,7 @@ void DocbookGenerator::init()
   QDir d(dir);
   if (!d.exists() && !d.mkdir(dir))
   {
-    err("Could not create output directory %s\n",dir.data());
-    exit(1);
+    term("Could not create output directory %s\n",dir.data());
   }
 
   createSubDirs(d);
@@ -537,7 +542,7 @@ DB_GEN_C2("IndexSections " << is)
       {
         t << "</title>" << endl;
         ClassSDict::Iterator cli(*Doxygen::classSDict);
-        ClassDef *cd=0;
+        const ClassDef *cd=0;
         bool found=FALSE;
         for (cli.toFirst();(cd=cli.current()) && !found;++cli)
         {
@@ -567,13 +572,9 @@ DB_GEN_C2("IndexSections " << is)
       {
         t << "</title>" << endl;
         bool isFirst=TRUE;
-        FileNameListIterator fnli(*Doxygen::inputNameList);
-        FileName *fn;
-        for (fnli.toFirst();(fn=fnli.current());++fnli)
+        for (const auto &fn : *Doxygen::inputNameLinkedMap)
         {
-          FileNameIterator fni(*fn);
-          FileDef *fd;
-          for (;(fd=fni.current());++fni)
+          for (const auto &fd : *fn)
           {
             if (fd->isLinkableInProject())
             {
@@ -635,7 +636,7 @@ DB_GEN_C
     if (!pd->getGroupDef() && !pd->isReference() && pd->name() == stripPath(name))
     {
       t << "<chapter>\n";
-      if (!pd->title().isEmpty())
+      if (pd->hasTitle())
       {
         t << "    <title>" << convertToDocBook(pd->title()) << "</title>" << endl;
       }
@@ -648,7 +649,8 @@ DB_GEN_C
     }
   }
 }
-void DocbookGenerator::writeDoc(DocNode *n,Definition *ctx,MemberDef *)
+
+void DocbookGenerator::writeDoc(DocNode *n,const Definition *,const MemberDef *)
 {
 DB_GEN_C
   DocbookDocVisitor *visitor =
@@ -673,7 +675,7 @@ void DocbookGenerator::writeString(const char *text)
 DB_GEN_C
   t << text;
 }
-void DocbookGenerator::startMemberHeader(const char *name,int)
+void DocbookGenerator::startMemberHeader(const char *,int)
 {
 DB_GEN_C
   t << "<simplesect>" << endl;
@@ -691,7 +693,7 @@ void DocbookGenerator::docify(const char *str)
 DB_GEN_C
   t << convertToDocBook(str);
 }
-void DocbookGenerator::writeObjectLink(const char *ref, const char *f,
+void DocbookGenerator::writeObjectLink(const char *, const char *f,
                                      const char *anchor, const char *text)
 {
 DB_GEN_C
@@ -806,7 +808,7 @@ DB_GEN_C
     t << "<programlisting>";
   }
 }
-void DocbookGenerator::endTextBlock(bool dense)
+void DocbookGenerator::endTextBlock(bool)
 {
 DB_GEN_C
   if (m_denseText)
@@ -815,8 +817,8 @@ DB_GEN_C
     t << "</programlisting>";
   }
 }
-void DocbookGenerator::startMemberDoc(const char *clname, const char *memname, const char *anchor, const char *title,
-                                      int memCount, int memTotal, bool showInline)
+void DocbookGenerator::startMemberDoc(const char *clname, const char *memname, const char *, const char *title,
+                                      int memCount, int memTotal, bool)
 {
 DB_GEN_C2("m_inLevel " << m_inLevel)
   t << "    <section>" << endl;
@@ -842,15 +844,15 @@ void DocbookGenerator::startTitleHead(const char *)
 DB_GEN_C
   t << "<title>";
 }
-void DocbookGenerator::endTitleHead(const char *fileName,const char *name)
+void DocbookGenerator::endTitleHead(const char *,const char *name)
 {
 DB_GEN_C
   t << "</title>" << endl;
   if (name) addIndexTerm(t, name);
 }
-void DocbookGenerator::startDoxyAnchor(const char *fName,const char *manName,
-                                 const char *anchor,const char *name,
-                                 const char *args)
+void DocbookGenerator::startDoxyAnchor(const char *fName,const char *,
+                                 const char *anchor,const char *,
+                                 const char *)
 {
 DB_GEN_C
   if (!m_inListItem[m_levelListItem] && !m_descTable)
@@ -863,7 +865,7 @@ DB_GEN_C
     t << "<anchor xml:id=\"_" << stripPath(fName) << "_1" << anchor << "\"/>";
   }
 }
-void DocbookGenerator::endDoxyAnchor(const char *fileName,const char *anchor)
+void DocbookGenerator::endDoxyAnchor(const char *,const char *)
 {
 DB_GEN_C
 }
@@ -876,7 +878,7 @@ void DocbookGenerator::endMemberDocName()
 {
 DB_GEN_C
 }
-void DocbookGenerator::startMemberGroupHeader(bool hasHeader)
+void DocbookGenerator::startMemberGroupHeader(bool)
 {
 DB_GEN_C
   t << "<simplesect><title>";
@@ -907,10 +909,10 @@ DB_GEN_C
   t << "    <informalfigure>" << endl;
   t << "        <mediaobject>" << endl;
   t << "            <imageobject>" << endl;
-  t << "                <imagedata width=\"50%\" align=\"center\" valign=\"middle\" scalefit=\"0\" fileref=\"" 
+  t << "                <imagedata width=\"50%\" align=\"center\" valign=\"middle\" scalefit=\"0\" fileref=\""
                          << relPath << fileName << ".png\">" << "</imagedata>" << endl;
   t << "            </imageobject>" << endl;
-  d.writeImage(t,dir,relPath,fileName,FALSE);
+  d.writeImage(t,m_dir,relPath,fileName,FALSE);
   t << "        </mediaobject>" << endl;
   t << "    </informalfigure>" << endl;
   t << "</para>" << endl;
@@ -1002,6 +1004,9 @@ DB_GEN_C
 void DocbookGenerator::endCodeFragment()
 {
 DB_GEN_C
+  //endCodeLine checks is there is still an open code line, if so closes it.
+  endCodeLine();
+
     t << "</programlisting>";
 }
 void DocbookGenerator::startMemberTemplateParams()
@@ -1015,13 +1020,13 @@ DB_GEN_C
   t << "</para>";
   t << "<para>";
 }
-void DocbookGenerator::startSection(const char *lab,const char *,SectionInfo::SectionType type)
+void DocbookGenerator::startSection(const char *lab,const char *,SectionType)
 {
 DB_GEN_C
   t << "    <section xml:id=\"_" << stripPath(lab) << "\">";
   t << "<title>";
 }
-void DocbookGenerator::endSection(const char *lab,SectionInfo::SectionType)
+void DocbookGenerator::endSection(const char *,SectionType)
 {
 DB_GEN_C
   t << "</title>";
@@ -1095,46 +1100,46 @@ void DocbookGenerator::startGroupCollaboration()
 {
 DB_GEN_C
 }
-void DocbookGenerator::endGroupCollaboration(const DotGroupCollaboration &g)
+void DocbookGenerator::endGroupCollaboration(DotGroupCollaboration &g)
 {
 DB_GEN_C
-  g.writeGraph(t,GOF_BITMAP,EOF_DocBook,Config_getString(DOCBOOK_OUTPUT),fileName,relPath,FALSE);
+  g.writeGraph(t,GOF_BITMAP,EOF_DocBook,Config_getString(DOCBOOK_OUTPUT),m_fileName,relPath,FALSE);
 }
 void DocbookGenerator::startDotGraph()
 {
 DB_GEN_C
 }
-void DocbookGenerator::endDotGraph(const DotClassGraph &g)
+void DocbookGenerator::endDotGraph(DotClassGraph &g)
 {
 DB_GEN_C
-  g.writeGraph(t,GOF_BITMAP,EOF_DocBook,Config_getString(DOCBOOK_OUTPUT),fileName,relPath,TRUE,FALSE);
+  g.writeGraph(t,GOF_BITMAP,EOF_DocBook,Config_getString(DOCBOOK_OUTPUT),m_fileName,relPath,TRUE,FALSE);
 }
 void DocbookGenerator::startInclDepGraph()
 {
 DB_GEN_C
 }
-void DocbookGenerator::endInclDepGraph(const DotInclDepGraph &g)
+void DocbookGenerator::endInclDepGraph(DotInclDepGraph &g)
 {
 DB_GEN_C
-  QCString fn = g.writeGraph(t,GOF_BITMAP,EOF_DocBook,Config_getString(DOCBOOK_OUTPUT), fileName,relPath,FALSE);
+  QCString fn = g.writeGraph(t,GOF_BITMAP,EOF_DocBook,Config_getString(DOCBOOK_OUTPUT), m_fileName,relPath,FALSE);
 }
 void DocbookGenerator::startCallGraph()
 {
 DB_GEN_C
 }
-void DocbookGenerator::endCallGraph(const DotCallGraph &g)
+void DocbookGenerator::endCallGraph(DotCallGraph &g)
 {
 DB_GEN_C
-  QCString fn = g.writeGraph(t,GOF_BITMAP,EOF_DocBook,Config_getString(DOCBOOK_OUTPUT), fileName,relPath,FALSE);
+  QCString fn = g.writeGraph(t,GOF_BITMAP,EOF_DocBook,Config_getString(DOCBOOK_OUTPUT), m_fileName,relPath,FALSE);
 }
 void DocbookGenerator::startDirDepGraph()
 {
 DB_GEN_C
 }
-void DocbookGenerator::endDirDepGraph(const DotDirDeps &g)
+void DocbookGenerator::endDirDepGraph(DotDirDeps &g)
 {
 DB_GEN_C
-  QCString fn = g.writeGraph(t,GOF_BITMAP,EOF_DocBook,Config_getString(DOCBOOK_OUTPUT), fileName,relPath,FALSE);
+  QCString fn = g.writeGraph(t,GOF_BITMAP,EOF_DocBook,Config_getString(DOCBOOK_OUTPUT), m_fileName,relPath,FALSE);
 }
 void DocbookGenerator::startMemberDocList()
 {
