@@ -17,67 +17,44 @@
 #define DOT_H
 
 #include <qlist.h>
-#include <qdict.h>
-#include <qwaitcondition.h>
-#include <qmutex.h>
-#include <qqueue.h>
-#include <qthread.h>
+#include <qcstring.h>
+#include <map>
+
 #include "sortdict.h"
-#include "qgstring.h"
-#include "qdir.h"
-#include "qcstring.h"
-#include "dotgraph.h"
+
+#include "dotgraph.h" // only for GraphOutputFormat
 #include "dotfilepatcher.h"
 #include "dotrunner.h"
 
 class FTextStream;
 class DotRunner;
 class DotRunnerQueue;
-class DotWorkerThread;
 
-/** Singleton that manages dot relation actions */
+/** Singleton that manages parallel dot invocations and patching files for embedding image maps */
 class DotManager
 {
   public:
     static DotManager *instance();
-    DotRunner* createRunner(const QCString& absDotName, const QCString& md5Hash);
-    int  addMap(const QCString &file,const QCString &mapFile,
-                const QCString &relPath,bool urlOnly,
-                const QCString &context,const QCString &label);
-    int addFigure(const QCString &file,const QCString &baseName,
-                  const QCString &figureName,bool heightCheck);
-    int addSVGConversion(const QCString &file,const QCString &relPath,
-               bool urlOnly,const QCString &context,bool zoomable,int graphId);
-    int addSVGObject(const QCString &file,const QCString &baseName,
-                     const QCString &figureNAme,const QCString &relPath);
-    bool run();
+    DotRunner*      createRunner(const std::string& absDotName, const std::string& md5Hash);
+    DotFilePatcher *createFilePatcher(const std::string &fileName);
+    bool run() const;
 
   private:
     DotManager();
     virtual ~DotManager();
 
-    QDict<DotRunner>       m_runners;
-    SDict<DotFilePatcher> m_dotMaps;
+    std::map<std::string, std::unique_ptr<DotRunner>>       m_runners;
+    std::map<std::string, DotFilePatcher>  m_filePatchers;
     static DotManager     *m_theInstance;
     DotRunnerQueue        *m_queue;
     QList<DotWorkerThread> m_workers;
 };
 
-void initDot();
-
-/** Generated a graphs legend page */
-void generateGraphLegend(const char *path);
-
 void writeDotGraphFromFile(const char *inFile,const char *outDir,
                            const char *outFile,GraphOutputFormat format);
 void writeDotImageMapFromFile(FTextStream &t,
                               const QCString& inFile, const QCString& outDir,
                               const QCString& relPath,const QCString& baseName,
                               const QCString& context,int graphId=-1);
-bool writeSVGFigureLink(FTextStream &out,const QCString &relPath,
-                        const QCString &baseName,const QCString &absImgName);
-bool convertMapFile(FTextStream &t,const char *mapName,
-                    const QCString relPath, bool urlOnly=FALSE,
-                    const QCString &context=QCString());
 
 #endif
