@@ -1,13 +1,10 @@
 /******************************************************************************
- *
- * 
- *
  *
  * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
- * documentation under the terms of the GNU General Public License is hereby 
- * granted. No representations are made about the suitability of this software 
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
  * for any purpose. It is provided "as is" without express or implied warranty.
  * See the GNU General Public License for more details.
  *
@@ -37,7 +34,6 @@ class PageSDict;
 class PageDef;
 class SearchIndexIntf;
 class ParserManager;
-class ObjCache;
 class Store;
 class QFileInfo;
 class BufStr;
@@ -49,9 +45,8 @@ class FileDef;
 class ClassDef;
 class ClassSDict;
 class GenericsSDict;
-class MemberNameSDict;
-class FileNameDict;
-class FileNameList;
+class MemberNameLinkedMap;
+class FileNameLinkedMap;
 class NamespaceSDict;
 class NamespaceDef;
 class DefinitionIntf;
@@ -61,7 +56,7 @@ class IndexList;
 class FormulaList;
 class FormulaDict;
 class FormulaNameDict;
-class SectionDict;
+class Preprocessor;
 struct MemberGroupInfo;
 
 typedef QList<QCString>    StringList;
@@ -71,7 +66,7 @@ typedef QListIterator<QCString>    StringListIterator;
 
 class StringDict : public QDict<QCString>
 {
-  public: 
+  public:
     StringDict(uint size=17) : QDict<QCString>(size) {}
     virtual ~StringDict() {}
 };
@@ -79,10 +74,10 @@ class StringDict : public QDict<QCString>
 struct LookupInfo
 {
   LookupInfo() : classDef(0), typeDef(0) {}
-  LookupInfo(ClassDef *cd,MemberDef *td,QCString ts,QCString rt) 
+  LookupInfo(const ClassDef *cd,const MemberDef *td,QCString ts,QCString rt)
     : classDef(cd), typeDef(td), templSpec(ts),resolvedType(rt) {}
-  ClassDef  *classDef;
-  MemberDef *typeDef;
+  const ClassDef  *classDef;
+  const MemberDef *typeDef;
   QCString   templSpec;
   QCString   resolvedType;
 };
@@ -102,31 +97,25 @@ class Doxygen
     static PageSDict                *pageSDict;
     static PageDef                  *mainPage;
     static bool                      insideMainPage;
-    static FileNameDict             *includeNameDict;
-    static FileNameDict             *exampleNameDict;
+    static FileNameLinkedMap        *includeNameLinkedMap;
+    static FileNameLinkedMap        *exampleNameLinkedMap;
     static QDict<void>               inputPaths;
-    static FileNameDict             *inputNameDict;
-    static FileNameList             *inputNameList;
-    static FileNameDict             *imageNameDict;
-    static FileNameDict             *dotFileNameDict;
-    static FileNameDict             *mscFileNameDict;
-    static FileNameDict             *diaFileNameDict;
+    static FileNameLinkedMap        *inputNameLinkedMap;
+    static FileNameLinkedMap        *imageNameLinkedMap;
+    static FileNameLinkedMap        *dotFileNameLinkedMap;
+    static FileNameLinkedMap        *mscFileNameLinkedMap;
+    static FileNameLinkedMap        *diaFileNameLinkedMap;
+    static MemberNameLinkedMap      *memberNameLinkedMap;
+    static MemberNameLinkedMap      *functionNameLinkedMap;
     static QStrList                  tagfileList;
-    static MemberNameSDict          *memberNameSDict;
-    static MemberNameSDict          *functionNameSDict;
-    static SectionDict              *sectionDict;
     static StringDict                namespaceAliasDict;
     static GroupSDict               *groupSDict;
     static NamespaceSDict           *namespaceSDict;
-    static FormulaList              *formulaList;
-    static FormulaDict              *formulaDict;
-    static FormulaDict              *formulaNameDict;
-    static StringDict                tagDestinationDict; 
-    static StringDict                aliasDict; 
+    static StringDict                tagDestinationDict;
+    static StringDict                aliasDict;
     static QIntDict<MemberGroupInfo> memGrpInfoDict;
     static QDict<void>               expandAsDefinedDict;
     static NamespaceDef             *globalScope;
-    static QDict<RefList>           *xrefLists; // array of xref lists: todo, test, bug, deprecated ...
     static QCString                  htmlFileExtension;
     static bool                      parseSourcesNeeded;
     static QTime                     runningTime;
@@ -143,8 +132,7 @@ class Doxygen
     static Store                    *symbolStorage;
     static QCString                  objDBFileName;
     static QCString                  entryDBFileName;
-    static CiteDict                 *citeDict;
-    static bool                      gatherDefines;
+    static QCString                  filterDBFileName;
     static bool                      userComments;
     static IndexList                *indexList;
     static int                       subpageNestingLevel;
@@ -152,6 +140,7 @@ class Doxygen
     static bool                      generatingXmlOutput;
     static bool                      markdownSupport;
     static GenericsSDict            *genericsDict;
+    static Preprocessor             *preprocessor;
 };
 
 void initDoxygen();
@@ -162,11 +151,10 @@ void searchInputFiles(StringList &inputFiles);
 void parseInput();
 void generateOutput();
 void readAliases();
-void readFormulaRepository();
+void readFormulaRepository(QCString dir, bool cmp = FALSE);
 void cleanUpDoxygen();
 int readFileOrDirectory(const char *s,
-                        FileNameList *fnList,
-                        FileNameDict *fnDict,
+                        FileNameLinkedMap *fnDict,
                         StringDict *exclDict,
                         QStrList *patList,
                         QStrList *exclPatList,
@@ -178,8 +166,7 @@ int readFileOrDirectory(const char *s,
                         QDict<void> *paths = 0
                        );
 int readDir(QFileInfo *fi,
-            FileNameList *fnList,
-            FileNameDict *fnDict,
+            FileNameLinkedMap *fnDict,
             StringDict  *exclDict,
             QStrList *patList,
             QStrList *exclPatList,
