@@ -43,10 +43,36 @@
 
 static int folderId=1;
 
+const char *JAVASCRIPT_LICENSE_TEXT = R"LIC(/*
+ @licstart  The following is the entire license notice for the JavaScript code in this file.
+
+ The MIT License (MIT)
+
+ Copyright (C) 1997-2020 by Dimitri van Heesch
+
+ Permission is hereby granted, free of charge, to any person obtaining a copy of this software
+ and associated documentation files (the "Software"), to deal in the Software without restriction,
+ including without limitation the rights to use, copy, modify, merge, publish, distribute,
+ sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
+ furnished to do so, subject to the following conditions:
+
+ The above copyright notice and this permission notice shall be included in all copies or
+ substantial portions of the Software.
+
+ THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
+ BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+ DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
+ @licend  The above is the entire license notice for the JavaScript code in this file
+*/
+)LIC";
+
 struct FTVNode
 {
   FTVNode(bool dir,const char *r,const char *f,const char *a,
-          const char *n,bool sepIndex,bool navIndex,Definition *df)
+          const char *n,bool sepIndex,bool navIndex,const Definition *df)
     : isLast(TRUE), isDir(dir),ref(r),file(f),anchor(a),name(n), index(0),
       parent(0), separateIndex(sepIndex), addToNavIndex(navIndex),
       def(df) { children.setAutoDelete(TRUE); }
@@ -63,7 +89,7 @@ struct FTVNode
   FTVNode *parent;
   bool separateIndex;
   bool addToNavIndex;
-  Definition *def;
+  const Definition *def;
 };
 
 int FTVNode::computeTreeDepth(int level) const
@@ -187,7 +213,7 @@ void FTVHelp::addContentsItem(bool isDir,
                               const char *anchor,
                               bool separateIndex,
                               bool addToNavIndex,
-                              Definition *def
+                              const Definition *def
                               )
 {
   //printf("%p: m_indent=%d addContentsItem(%s,%s,%s,%s)\n",this,m_indent,name,ref,file,anchor);
@@ -207,7 +233,7 @@ void FTVHelp::addContentsItem(bool isDir,
 
 }
 
-static QCString node2URL(FTVNode *n,bool overruleFile=FALSE,bool srcLink=FALSE)
+static QCString node2URL(const FTVNode *n,bool overruleFile=FALSE,bool srcLink=FALSE)
 {
   QCString url = n->file;
   if (!url.isEmpty() && url.at(0)=='!')  // relative URL
@@ -223,7 +249,7 @@ static QCString node2URL(FTVNode *n,bool overruleFile=FALSE,bool srcLink=FALSE)
   {
     if (overruleFile && n->def && n->def->definitionType()==Definition::TypeFile)
     {
-      FileDef *fd = (FileDef*)n->def;
+      const FileDef *fd = dynamic_cast<const FileDef*>(n->def);
       if (srcLink)
       {
         url = fd->getSourceFileBase();
@@ -233,7 +259,7 @@ static QCString node2URL(FTVNode *n,bool overruleFile=FALSE,bool srcLink=FALSE)
         url = fd->getOutputFileBase();
       }
     }
-    url+=Doxygen::htmlFileExtension;
+    url = addHtmlExtensionIfMissing(url);
     if (!n->anchor.isEmpty()) url+="#"+n->anchor;
   }
   return url;
@@ -286,7 +312,7 @@ void FTVHelp::generateLink(FTextStream &t,FTVNode *n)
       t << "<a class=\"elRef\" ";
       QCString result = externalLinkTarget();
       if (result != "") setTarget = TRUE;
-      t << result << externalRef("",n->ref,FALSE);
+      t << result;
     }
     else // local link
     {
@@ -315,7 +341,7 @@ void FTVHelp::generateLink(FTextStream &t,FTVNode *n)
   }
 }
 
-static void generateBriefDoc(FTextStream &t,Definition *def)
+static void generateBriefDoc(FTextStream &t,const Definition *def)
 {
   QCString brief = def->briefDescription(TRUE);
   //printf("*** %p: generateBriefDoc(%s)='%s'\n",def,def->name().data(),brief.data());
@@ -332,7 +358,7 @@ static void generateBriefDoc(FTextStream &t,Definition *def)
   }
 }
 
-static char compoundIcon(ClassDef *cd)
+static char compoundIcon(const ClassDef *cd)
 {
   char icon='C';
   if (cd->getLanguage() == SrcLangExt_Slice)
@@ -392,7 +418,7 @@ void FTVHelp::generateTree(FTextStream &t, const QList<FTVNode> &nl,int level,in
       }
       else if (n->def && n->def->definitionType()==Definition::TypeClass)
       {
-        char icon=compoundIcon(dynamic_cast<ClassDef*>(n->def));
+        char icon=compoundIcon(dynamic_cast<const ClassDef*>(n->def));
         t << "<span class=\"icona\"><span class=\"icon\">" << icon << "</span></span>";
       }
       else
@@ -415,11 +441,11 @@ void FTVHelp::generateTree(FTextStream &t, const QList<FTVNode> &nl,int level,in
     }
     else // leaf node
     {
-      FileDef *srcRef=0;
+      const FileDef *srcRef=0;
       if (n->def && n->def->definitionType()==Definition::TypeFile &&
-          ((FileDef*)n->def)->generateSourceFile())
+          (dynamic_cast<const FileDef*>(n->def))->generateSourceFile())
       {
-        srcRef = (FileDef*)n->def;
+        srcRef = dynamic_cast<const FileDef*>(n->def);
       }
       if (srcRef)
       {
@@ -448,9 +474,13 @@ void FTVHelp::generateTree(FTextStream &t, const QList<FTVNode> &nl,int level,in
       }
       else if (n->def && n->def->definitionType()==Definition::TypeClass)
       {
-        char icon=compoundIcon(dynamic_cast<ClassDef*>(n->def));
+        char icon=compoundIcon(dynamic_cast<const ClassDef*>(n->def));
         t << "<span class=\"icona\"><span class=\"icon\">" << icon << "</span></span>";
       }
+      else if (n->def && n->def->definitionType()==Definition::TypeDir)
+      {
+        t << "<span class=\"iconfclosed\"></span>";
+      }
       else
       {
         t << "<span class=\"icondoc\"></span>";
@@ -492,7 +522,7 @@ class NavIndexEntryList : public QList<NavIndexEntry>
     }
 };
 
-static QCString pathToNode(FTVNode *leaf,FTVNode *n)
+static QCString pathToNode(const FTVNode *leaf,const FTVNode *n)
 {
   QCString result;
   if (n->parent)
@@ -511,7 +541,7 @@ static bool dupOfParent(const FTVNode *n)
   return FALSE;
 }
 
-static void generateJSLink(FTextStream &t,FTVNode *n)
+static void generateJSLink(FTextStream &t,const FTVNode *n)
 {
   if (n->file.isEmpty()) // no link
   {
@@ -542,7 +572,7 @@ static bool generateJSTree(NavIndexEntryList &navIndex,FTextStream &t,
   indentStr.fill(' ',level*2);
   bool found=FALSE;
   QListIterator<FTVNode> nli(nl);
-  FTVNode *n;
+  const FTVNode *n;
   for (nli.toFirst();(n=nli.current());++nli)
   {
     // terminate previous entry
@@ -560,7 +590,7 @@ static bool generateJSTree(NavIndexEntryList &navIndex,FTextStream &t,
     {
       if (n->def && n->def->definitionType()==Definition::TypeFile)
       {
-        FileDef *fd = (FileDef*)n->def;
+        const FileDef *fd = dynamic_cast<const FileDef*>(n->def);
         bool doc,src;
         doc = fileVisibleInIndex(fd,src);
         if (doc)
@@ -635,29 +665,14 @@ static void generateJSNavTree(const QList<FTVNode> &nodeList)
     //tidx << "var NAVTREEINDEX =" << endl;
     //tidx << "{" << endl;
     FTextStream t(&f);
-		t << "/*\n@ @licstart  The following is the entire license notice for the\n"
-			"JavaScript code in this file.\n\nCopyright (C) 1997-2017 by Dimitri van Heesch\n\n"
-			"This program is free software; you can redistribute it and/or modify\n"
-			"it under the terms of the GNU General Public License as published by\n"
-			"the Free Software Foundation; either version 2 of the License, or\n"
-			"(at your option) any later version.\n\n"
-			"This program is distributed in the hope that it will be useful,\n"
-			"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
-			" MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
-			" GNU General Public License for more details.\n\n"
-			"You should have received a copy of the GNU General Public License along\n"
-			"with this program; if not, write to the Free Software Foundation, Inc.,\n"
-			"51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.\n\n"
-			"@licend  The above is the entire license notice\n"
-			"for the JavaScript code in this file\n"
-			"*/\n";
+    t << JAVASCRIPT_LICENSE_TEXT;
     t << "var NAVTREE =" << endl;
     t << "[" << endl;
     t << "  [ ";
     QCString &projName = Config_getString(PROJECT_NAME);
     if (projName.isEmpty())
     {
-      if (Doxygen::mainPage && !Doxygen::mainPage->title().isEmpty()) // Use title of main page as root
+      if (mainPageHasTitle()) // Use title of main page as root
       {
         t << "\"" << convertToJSString(Doxygen::mainPage->title()) << "\", ";
       }
@@ -704,7 +719,7 @@ static void generateJSNavTree(const QList<FTVNode> &nodeList)
       tsidx << "{" << endl;
       QListIterator<NavIndexEntry> li(navIndex);
       NavIndexEntry *e;
-      bool first=TRUE;
+      first=TRUE;
       for (li.toFirst();(e=li.current());) // for each entry
       {
         if (elemCount==0)
@@ -796,8 +811,7 @@ void FTVHelp::generateTreeViewInline(FTextStream &t)
     t << "<div class=\"levels\">[";
     t << theTranslator->trDetailLevel();
     t << " ";
-    int i;
-    for (i=1;i<=depth;i++)
+    for (int i=1;i<=depth;i++)
     {
       t << "<span onclick=\"javascript:toggleLevel(" << i << ");\">" << i << "</span>";
     }
@@ -809,9 +823,7 @@ void FTVHelp::generateTreeViewInline(FTextStream &t)
       for (int i=1;i<=depth;i++)
       {
         int num=0;
-        QListIterator<FTVNode> li(m_indentNodes[0]);
-        FTVNode *n;
-        for (;(n=li.current());++li)
+        for (li.toFirst();(n=li.current());++li)
         {
           num+=n->numNodesAtLevel(0,i);
         }
