@@ -26,7 +26,6 @@
 #include "groupdef.h"
 #include "doxygen.h"
 #include "docparser.h"
-#include "marshal.h"
 #include "entry.h"
 #include "md5.h"
 
@@ -41,24 +40,23 @@ MemberGroup::MemberGroup()
 {
 }
 
-MemberGroup::MemberGroup(Definition *parent,
-      int id,const char *hdr,const char *d,const char *docFile,int docLine)
+MemberGroup::MemberGroup(int id,const char *hdr,const char *d,const char *docFile,int docLine)
 {
+  static bool sortBriefDocs = Config_getBool(SORT_BRIEF_DOCS);
+
   //printf("New member group id=%d header=%s desc=%s\n",id,hdr,d);
   memberList      = new MemberList(MemberListType_memberGroup);
+  memberList->setNeedsSorting(sortBriefDocs); // detailed sections are already sorted elsewhere.
   grpId           = id;
   grpHeader       = hdr;
   doc             = d;
-  scope           = 0;
   inSameSection   = TRUE;
   inDeclSection   = 0;
   m_numDecMembers = -1;
   m_numDocMembers = -1;
-  m_parent        = parent;
   m_docFile       = docFile;
   m_docLine       = docLine;
-  m_xrefListItems = 0;
-  //printf("Member group docs=`%s'\n",doc.data());
+  //printf("Member group docs='%s'\n",doc.data());
 }
 
 MemberGroup::~MemberGroup()
@@ -68,33 +66,32 @@ MemberGroup::~MemberGroup()
 
 void MemberGroup::insertMember(MemberDef *md)
 {
-  //printf("MemberGroup::insertMember m_parent=%s memberList=%p count=%d"
+  //printf("MemberGroup::insertMember memberList=%p count=%d"
   //       " member section list: %p: md=%p:%s\n",
-  //       m_parent ? m_parent->name().data() : "<null>",
-  //       memberList->first() ? memberList->first()->getSectionList(m_parent) : 0,
+  //       memberList->first() ? memberList->first()->getSectionList() : 0,
   //       memberList->count(),
-  //       md->getSectionList(m_parent),
+  //       md->getSectionList(),
   //       md,md->name().data());
 
   MemberDef *firstMd = memberList->getFirst();
-  if (inSameSection && firstMd &&
-      firstMd->getSectionList(m_parent)!=md->getSectionList(m_parent))
+  if (inSameSection && firstMd && firstMd->getSectionList()!=md->getSectionList())
   {
     inSameSection=FALSE;
   }
   else if (inDeclSection==0)
   {
-    inDeclSection = md->getSectionList(m_parent);
+    inDeclSection = const_cast<MemberList*>(md->getSectionList());
     //printf("inDeclSection=%p type=%d\n",inDeclSection,inDeclSection->listType());
   }
   memberList->append(md);
 
   // copy the group of the first member in the memberGroup
   GroupDef *gd;
-  if (firstMd && (gd=firstMd->getGroupDef()))
+  if (firstMd && !firstMd->isAlias() && (gd=const_cast<GroupDef*>(firstMd->getGroupDef())))
   {
-    md->setGroupDef(gd, firstMd->getGroupPri(), 
-                    firstMd->getGroupFileName(), firstMd->getGroupStartLine(), 
+    md->setGroupDef(gd, firstMd->getGroupPri(),
+                    firstMd->getGroupFileName(),
+                    firstMd->getGroupStartLine(),
                     firstMd->getGroupHasDocs());
     gd->insertMember(md);
   }
@@ -107,8 +104,8 @@ void MemberGroup::setAnchors()
 }
 
 void MemberGroup::writeDeclarations(OutputList &ol,
-               ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd,
-               bool showInline)
+               const ClassDef *cd,const NamespaceDef *nd,const FileDef *fd,const GroupDef *gd,
+               bool showInline) const
 {
   //printf("MemberGroup::writeDeclarations() %s\n",grpHeader.data());
   QCString ldoc = doc;
@@ -117,42 +114,48 @@ void MemberGroup::writeDeclarations(OutputList &ol,
 }
 
 void MemberGroup::writePlainDeclarations(OutputList &ol,
-               ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd,
-               ClassDef *inheritedFrom,const char *inheritId
-              )
+               const ClassDef *cd,const NamespaceDef *nd,const FileDef *fd,const GroupDef *gd,
+               const ClassDef *inheritedFrom,const char *inheritId
+              ) const
 {
   //printf("MemberGroup::writePlainDeclarations() memberList->count()=%d\n",memberList->count());
   memberList->writePlainDeclarations(ol,cd,nd,fd,gd,inheritedFrom,inheritId);
 }
 
 void MemberGroup::writeDocumentation(OutputList &ol,const char *scopeName,
-               Definition *container,bool showEnumValues,bool showInline)
+               const Definition *container,bool showEnumValues,bool showInline) const
 {
   memberList->writeDocumentation(ol,scopeName,container,0,showEnumValues,showInline);
 }
 
 void MemberGroup::writeDocumentationPage(OutputList &ol,const char *scopeName,
-               Definition *container)
+               const Definition *container) const
 {
   memberList->writeDocumentationPage(ol,scopeName,container);
 }
 
-void MemberGroup::addGroupedInheritedMembers(OutputList &ol,ClassDef *cd,
+void MemberGroup::setAnonymousEnumType()
+{
+  memberList->setAnonymousEnumType();
+}
+
+void MemberGroup::addGroupedInheritedMembers(OutputList &ol,const ClassDef *cd,
                MemberListType lt,
-               ClassDef *inheritedFrom,const QCString &inheritId)
+               const ClassDef *inheritedFrom,const QCString &inheritId) const
 {
   //printf("** addGroupedInheritedMembers()\n");
   MemberListIterator li(*memberList);
   MemberDef *md;
   for (li.toFirst();(md=li.current());++li)
   {
-    //printf("matching %d == %d\n",lt,md->getSectionList(m_parent)->listType());
-    MemberList *ml = md->getSectionList(m_parent);
+    //printf("matching %d == %d\n",lt,md->getSectionList()->listType());
+    const MemberList *ml = md->getSectionList();
     if (ml && lt==ml->listType())
     {
-      MemberList ml(lt);
-      ml.append(md);
-      ml.writePlainDeclarations(ol,cd,0,0,0,inheritedFrom,inheritId);
+      MemberList mml(lt);
+      mml.append(md);
+      mml.countDecMembers();
+      mml.writePlainDeclarations(ol,cd,0,0,0,inheritedFrom,inheritId);
     }
   }
 }
@@ -165,8 +168,8 @@ int MemberGroup::countGroupedInheritedMembers(MemberListType lt)
   MemberDef *md;
   for (li.toFirst();(md=li.current());++li)
   {
-    //printf("matching %d == %d\n",lt,md->getSectionList(m_parent)->listType());
-    MemberList *ml = md->getSectionList(m_parent);
+    //printf("matching %d == %d\n",lt,md->getSectionList()->listType());
+    const MemberList *ml = md->getSectionList();
     if (ml && lt==ml->listType())
     {
       count++;
@@ -183,33 +186,24 @@ void MemberGroup::addToDeclarationSection()
 {
   if (inDeclSection)
   {
-    //printf("Adding group %p to list %p (type=%d)\n",this,
-    //                              inDeclSection,inDeclSection->listType());
+    //printf("Adding group %p to list %p (type=%d) memberList=%p\n",this,
+    //                             inDeclSection,inDeclSection->listType(),memberList);
     inDeclSection->addMemberGroup(this);
   }
 }
 
-int MemberGroup::countDecMembers(GroupDef *gd)
+void MemberGroup::countDecMembers()
 {
-  if (m_numDecMembers==-1) /* number of member not cached */
-  {
-    memberList->countDecMembers(gd);
-    m_numDecMembers = memberList->numDecMembers();
-  }
-  return m_numDecMembers;
+  memberList->countDecMembers();
 }
 
-int MemberGroup::countDocMembers()
+void MemberGroup::countDocMembers()
 {
-  if (m_numDocMembers==-1)
-  {
-    memberList->countDocMembers();
-    m_numDocMembers = memberList->numDocMembers();
-  }
-  return m_numDocMembers;
+  memberList->countDocMembers();
 }
 
-int MemberGroup::countInheritableMembers(ClassDef *inheritedFrom) const
+
+int MemberGroup::countInheritableMembers(const ClassDef *inheritedFrom) const
 {
   return memberList->countInheritableMembers(inheritedFrom);
 }
@@ -253,6 +247,7 @@ void MemberGroup::distributeMemberGroupDocumentation()
   }
 }
 
+#if 0
 int MemberGroup::varCount() const
 {
   return memberList->varCount();
@@ -302,15 +297,26 @@ int MemberGroup::friendCount() const
 { 
   return memberList->friendCount(); 
 }
+#endif
 
-int MemberGroup::numDecMembers() const  
-{ 
-  return memberList->numDecMembers(); 
+int MemberGroup::numDecMembers() const
+{
+  return memberList->numDecMembers();
 }
 
-int MemberGroup::numDocMembers() const  
-{ 
-  return memberList->numDocMembers(); 
+int MemberGroup::numDecEnumValues() const
+{
+  return memberList->numDecEnumValues();
+}
+
+int MemberGroup::numDocMembers() const
+{
+  return memberList->numDocMembers();
+}
+
+int MemberGroup::numDocEnumValues() const
+{
+  return memberList->numDocEnumValues();
 }
 
 void MemberGroup::setInGroup(bool b)
@@ -318,6 +324,7 @@ void MemberGroup::setInGroup(bool b)
   memberList->setInGroup(b);
 }
 
+
 QCString MemberGroup::anchor() const
 {
   uchar md5_sig[16];
@@ -332,7 +339,7 @@ QCString MemberGroup::anchor() const
 void MemberGroup::addListReferences(Definition *def)
 {
   memberList->addListReferences(def);
-  if (m_xrefListItems && def)
+  if (def)
   {
     QCString name = def->getOutputFileBase()+"#"+anchor();
     addRefItem(m_xrefListItems,
@@ -343,63 +350,15 @@ void MemberGroup::addListReferences(Definition *def)
   }
 }
 
-void MemberGroup::findSectionsInDocumentation()
+void MemberGroup::findSectionsInDocumentation(const Definition *d)
 {
-  docFindSections(doc,0,this,m_docFile);
-  memberList->findSectionsInDocumentation();
+  docFindSections(doc,d,m_docFile);
+  memberList->findSectionsInDocumentation(d);
 }
 
-void MemberGroup::marshal(StorageIntf *s)
+void MemberGroup::setRefItems(const std::vector<RefItem*> &sli)
 {
-  marshalMemberList(s,memberList);
-  marshalObjPointer(s,inDeclSection); // reference only
-  marshalInt(s,grpId);
-  marshalQCString(s,grpHeader);
-  marshalQCString(s,fileName);
-  marshalObjPointer(s,scope);
-  marshalQCString(s,doc);
-  marshalBool(s,inSameSection);
-  marshalInt(s,m_numDecMembers);
-  marshalInt(s,m_numDocMembers);
-  marshalObjPointer(s,m_parent);
-  marshalQCString(s,m_docFile);
-  marshalItemInfoList (Doxygen::symbolStorage,m_xrefListItems);
-}
-
-void MemberGroup::unmarshal(StorageIntf *s)
-{
-  memberList      = unmarshalMemberList(s);
-  inDeclSection   = (MemberList *)unmarshalObjPointer(s); 
-  grpId           = unmarshalInt(s);
-  grpHeader       = unmarshalQCString(s);
-  fileName        = unmarshalQCString(s);
-  scope           = (Definition *)unmarshalObjPointer(s);
-  doc             = unmarshalQCString(s);
-  inSameSection   = unmarshalBool(s);
-  m_numDecMembers = unmarshalInt(s);
-  m_numDocMembers = unmarshalInt(s);
-  m_parent        = (Definition *)unmarshalObjPointer(s);
-  m_docFile       = unmarshalQCString(s);
-  m_xrefListItems = unmarshalItemInfoList (Doxygen::symbolStorage);
-}
-
-void MemberGroup::setRefItems(const QList<ListItemInfo> *sli)
-{
-  if (sli)
-  {
-    // deep copy the list
-    if (m_xrefListItems==0) 
-    {
-      m_xrefListItems=new QList<ListItemInfo>;
-      m_xrefListItems->setAutoDelete(TRUE);
-    }
-    QListIterator<ListItemInfo> slii(*sli);
-    ListItemInfo *lii;
-    for (slii.toFirst();(lii=slii.current());++slii)
-    {
-      m_xrefListItems->append(new ListItemInfo(*lii));
-    } 
-  }
+  m_xrefListItems.insert(m_xrefListItems.end(), sli.cbegin(), sli.cend());
 }
 
 void MemberGroup::writeTagFile(FTextStream &tagFile)
@@ -409,18 +368,7 @@ void MemberGroup::writeTagFile(FTextStream &tagFile)
 
 //--------------------------------------------------------------------------
 
-void MemberGroupInfo::setRefItems(const QList<ListItemInfo> *sli)
+void MemberGroupInfo::setRefItems(const std::vector<RefItem*> &sli)
 {
-  if (!sli) return;
-  if (m_sli==0)
-  {
-    m_sli = new QList<ListItemInfo>;
-    m_sli->setAutoDelete(TRUE);
-  }
-  QListIterator<ListItemInfo> slii(*sli);
-  ListItemInfo *ili;
-  for (slii.toFirst();(ili=slii.current());++slii)
-  {
-    m_sli->append(new ListItemInfo(*ili));
-  }
+  m_sli.insert(m_sli.end(), sli.cbegin(), sli.cend());
 }
