@@ -1,12 +1,12 @@
 /******************************************************************************
  *
- * $Id: membername.h,v 1.17 2001/03/19 19:27:41 root Exp $
  *
- * Copyright (C) 1997-2003 by Dimitri van Heesch.
+ *
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
- * documentation under the terms of the GNU General Public License is hereby 
- * granted. No representations are made about the suitability of this software 
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
  * for any purpose. It is provided "as is" without express or implied warranty.
  * See the GNU General Public License for more details.
  *
@@ -21,34 +21,23 @@
 #include <qlist.h>
 #include "memberdef.h"
 #include "sortdict.h"
+#include "linkedmap.h"
 
-class MemberName : public QList<MemberDef>
-{ 
+class MemberName : public std::vector< std::unique_ptr<MemberDef> >
+{
   public:
-    MemberName(const char *name);
-   ~MemberName();
-    const char *memberName() const { return name; }
-   
-    int compareItems(GCI item1,GCI item2);
+    MemberName(const char *name) : m_name(name) {}
+    const char *memberName() const { return m_name; }
   private:
-    QCString name;
+    QCString m_name;
 };
 
-class MemberNameIterator : public QListIterator<MemberDef>
+/** Ordered dictionary of MemberName objects. */
+class MemberNameLinkedMap : public LinkedMap<MemberName>
 {
-  public:
-    MemberNameIterator( const MemberName &list);
-};
-
-class MemberNameSDict : public SDict<MemberName>
-{
-  public:
-    MemberNameSDict(int size) : SDict<MemberName>(size) {}
-   ~MemberNameSDict() {}
-
-   int compareItems(GCI item1,GCI item2);
 };
 
+/** Data associated with a MemberDef in an inheritance relation. */
 struct MemberInfo
 {
   MemberInfo(MemberDef *md,Protection p,Specifier v,bool inh) :
@@ -59,39 +48,40 @@ struct MemberInfo
   Specifier  virt;
   bool       inherited;
   QCString   scopePath;
-  QCString   ambiguityResolutionScope; 
+  QCString   ambiguityResolutionScope;
   ClassDef  *ambigClass;
 };
 
+/** Class representing all MemberInfo objects with the same name */
 class MemberNameInfo : public QList<MemberInfo>
 {
   public:
-    MemberNameInfo(const char *name);  
+    MemberNameInfo(const char *name);
    ~MemberNameInfo() {}
     const char *memberName() const { return name; }
-    int compareItems(GCI item1,GCI item2);
   private:
+    int compareValues(const MemberInfo *item1,const MemberInfo *item2) const;
     QCString name;
 };
 
+/** Iterator for MemberInfo objects in a MemberNameInfo list. */
 class MemberNameInfoIterator : public QListIterator<MemberInfo>
 {
   public:
-    MemberNameInfoIterator(const MemberNameInfo &mnii) 
+    MemberNameInfoIterator(const MemberNameInfo &mnii)
       : QListIterator<MemberInfo>(mnii) {}
 };
 
-
+/** Sorted dictionary of MemberNameInfo objects. */
 class MemberNameInfoSDict : public SDict<MemberNameInfo>
 {
   public:
-    MemberNameInfoSDict(int size) : SDict<MemberNameInfo>(size) {}
+    MemberNameInfoSDict(uint size) : SDict<MemberNameInfo>(size) {}
    ~MemberNameInfoSDict() {}
-    int compareItems(GCI item1,GCI item2) 
-    { return stricmp(
-                    ((MemberNameInfo *)item1)->memberName(),
-                    ((MemberNameInfo *)item2)->memberName()
-                   );
+  private:
+    int compareValues(const MemberNameInfo *item1,const MemberNameInfo *item2) const
+    {
+      return qstricmp(item1->memberName(), item2->memberName());
     }
 };
 
