@@ -1,13 +1,13 @@
 /******************************************************************************
  *
- * 
+ *
  *
  *
  * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
- * documentation under the terms of the GNU General Public License is hereby 
- * granted. No representations are made about the suitability of this software 
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
  * for any purpose. It is provided "as is" without express or implied warranty.
  * See the GNU General Public License for more details.
  *
@@ -21,6 +21,8 @@
 #include <assert.h>
 #include <stdio.h>
 #include <stdarg.h>
+#include <algorithm>
+
 
 #include <qxml.h>
 #include <qstack.h>
@@ -48,7 +50,7 @@ class TagAnchorInfo
   public:
     TagAnchorInfo(const QCString &f,
                   const QCString &l,
-                  const QCString &t=QCString()) 
+                  const QCString &t=QCString())
       : label(l), fileName(f), title(t) {}
     QCString label;
     QCString fileName;
@@ -58,7 +60,7 @@ class TagAnchorInfo
 /** List of TagAnchorInfo objects. */
 class TagAnchorInfoList : public QList<TagAnchorInfo>
 {
-  public: 
+  public:
     TagAnchorInfoList() : QList<TagAnchorInfo>() { setAutoDelete(TRUE); }
     virtual ~TagAnchorInfoList() {}
 };
@@ -77,7 +79,7 @@ class TagEnumValueInfo
 class TagMemberInfo
 {
   public:
-    TagMemberInfo() : prot(Public), virt(Normal), isStatic(FALSE) 
+    TagMemberInfo() : prot(Public), virt(Normal), isStatic(FALSE)
     { enumValues.setAutoDelete(TRUE); }
     QCString type;
     QCString name;
@@ -89,7 +91,7 @@ class TagMemberInfo
     TagAnchorInfoList docAnchors;
     Protection prot;
     Specifier virt;
-    bool isStatic; 
+    bool isStatic;
     QList<TagEnumValueInfo> enumValues;
 };
 
@@ -98,14 +100,14 @@ class TagClassInfo
 {
   public:
     enum Kind { None=-1, Class, Struct, Union, Interface, Exception, Protocol, Category, Enum, Service, Singleton };
-    TagClassInfo() { bases=0, templateArguments=0; members.setAutoDelete(TRUE); isObjC=FALSE; kind = None; }
-   ~TagClassInfo() { delete bases; delete templateArguments; }
+    TagClassInfo() { templateArguments=0; members.setAutoDelete(TRUE); isObjC=FALSE; kind = None; }
+   ~TagClassInfo() { delete templateArguments; }
     QCString name;
     QCString filename;
     QCString clangId;
     QCString anchor;
     TagAnchorInfoList docAnchors;
-    QList<BaseInfo> *bases;
+    std::vector<BaseInfo> bases;
     QList<TagMemberInfo> members;
     QList<QCString> *templateArguments;
     QCStringList classList;
@@ -205,10 +207,10 @@ class TagDirInfo
     TagAnchorInfoList docAnchors;
 };
 
-/** Tag file parser. 
+/** Tag file parser.
  *
  *  Reads an XML-structured tagfile and builds up the structure in
- *  memory. The method buildLists() is used to transfer/translate 
+ *  memory. The method buildLists() is used to transfer/translate
  *  the structures to the doxygen engine.
  */
 class TagFileParser : public QXmlDefaultHandler
@@ -227,7 +229,7 @@ class TagFileParser : public QXmlDefaultHandler
                };
     class StartElementHandler
     {
-        typedef void (TagFileParser::*Handler)(const QXmlAttributes &attrib); 
+        typedef void (TagFileParser::*Handler)(const QXmlAttributes &attrib);
       public:
         StartElementHandler(TagFileParser *parent, Handler h) : m_parent(parent), m_handler(h) {}
         void operator()(const QXmlAttributes &attrib) { (m_parent->*m_handler)(attrib); }
@@ -238,7 +240,7 @@ class TagFileParser : public QXmlDefaultHandler
 
     class EndElementHandler
     {
-        typedef void (TagFileParser::*Handler)(); 
+        typedef void (TagFileParser::*Handler)();
       public:
         EndElementHandler(TagFileParser *parent, Handler h) : m_parent(parent), m_handler(h) {}
         void operator()() { (m_parent->*m_handler)(); }
@@ -280,7 +282,7 @@ class TagFileParser : public QXmlDefaultHandler
 
     void warn(const char *fmt)
     {
-      ::warn(m_inputFileName,m_locator->lineNumber(),fmt);
+      ::warn(m_inputFileName,m_locator->lineNumber(),"%s", fmt);
     }
     void warn(const char *fmt,const char *s)
     {
@@ -389,7 +391,7 @@ class TagFileParser : public QXmlDefaultHandler
       }
       if (isObjC=="yes" && m_curClass)
       {
-        m_curClass->isObjC = TRUE; 
+        m_curClass->isObjC = TRUE;
       }
     }
 
@@ -397,20 +399,20 @@ class TagFileParser : public QXmlDefaultHandler
     {
       switch (m_state)
       {
-        case InClass:     m_tagFileClasses.append(m_curClass); 
-                          m_curClass=0; break; 
-        case InFile:      m_tagFileFiles.append(m_curFile); 
-                          m_curFile=0; break; 
-        case InNamespace: m_tagFileNamespaces.append(m_curNamespace); 
-                          m_curNamespace=0; break; 
-        case InGroup:     m_tagFileGroups.append(m_curGroup); 
-                          m_curGroup=0; break; 
-        case InPage:      m_tagFilePages.append(m_curPage); 
-                          m_curPage=0; break; 
+        case InClass:     m_tagFileClasses.append(m_curClass);
+                          m_curClass=0; break;
+        case InFile:      m_tagFileFiles.append(m_curFile);
+                          m_curFile=0; break;
+        case InNamespace: m_tagFileNamespaces.append(m_curNamespace);
+                          m_curNamespace=0; break;
+        case InGroup:     m_tagFileGroups.append(m_curGroup);
+                          m_curGroup=0; break;
+        case InPage:      m_tagFilePages.append(m_curPage);
+                          m_curPage=0; break;
         case InDir:       m_tagFileDirs.append(m_curDir);
                           m_curDir=0; break;
-        case InPackage:   m_tagFilePackages.append(m_curPackage); 
-                          m_curPackage=0; break; 
+        case InPackage:   m_tagFilePackages.append(m_curPackage);
+                          m_curPackage=0; break;
         default:
                           warn("tag 'compound' was not expected!");
       }
@@ -458,7 +460,7 @@ class TagFileParser : public QXmlDefaultHandler
         case InNamespace: m_curNamespace->members.append(m_curMember); break;
         case InGroup:     m_curGroup->members.append(m_curMember); break;
         case InPackage:   m_curPackage->members.append(m_curMember); break;
-        default:   warn("Unexpected tag 'member' found"); break; 
+        default:   warn("Unexpected tag 'member' found"); break;
       }
     }
 
@@ -482,7 +484,7 @@ class TagFileParser : public QXmlDefaultHandler
 
     void endEnumValue()
     {
-      m_curEnumValue->name = m_curString.stripWhiteSpace(); 
+      m_curEnumValue->name = m_curString.stripWhiteSpace();
       m_state = *m_stateStack.top();
       m_stateStack.remove();
       if (m_state==InMember)
@@ -494,6 +496,23 @@ class TagFileParser : public QXmlDefaultHandler
 
     void endDocAnchor()
     {
+      // Check whether or not the tag is automatically generate, in that case ignore the tag.
+      switch(m_state)
+      {
+        case InClass:
+        case InFile:
+        case InNamespace:
+        case InGroup:
+        case InPage:
+        case InMember:
+        case InPackage:
+        case InDir:
+          if (QString(m_curString).startsWith("autotoc_md")) return;
+          break;
+        default:
+          warn("Unexpected tag 'docanchor' found");
+          return;
+      }
       switch(m_state)
       {
         case InClass:     m_curClass->docAnchors.append(new TagAnchorInfo(m_fileName,m_curString,m_title)); break;
@@ -504,7 +523,7 @@ class TagFileParser : public QXmlDefaultHandler
         case InMember:    m_curMember->docAnchors.append(new TagAnchorInfo(m_fileName,m_curString,m_title)); break;
         case InPackage:   m_curPackage->docAnchors.append(new TagAnchorInfo(m_fileName,m_curString,m_title)); break;
         case InDir:       m_curDir->docAnchors.append(new TagAnchorInfo(m_fileName,m_curString,m_title)); break;
-        default:   warn("Unexpected tag 'docanchor' found"); break; 
+        default: break; // will not be reached
       }
     }
 
@@ -517,7 +536,7 @@ class TagFileParser : public QXmlDefaultHandler
         case InNamespace: m_curNamespace->classList.append(m_curString); break;
         case InGroup:     m_curGroup->classList.append(m_curString); break;
         case InPackage:   m_curPackage->classList.append(m_curString); break;
-        default:   warn("Unexpected tag 'class' found"); break; 
+        default:   warn("Unexpected tag 'class' found"); break;
       }
     }
 
@@ -528,7 +547,7 @@ class TagFileParser : public QXmlDefaultHandler
         case InNamespace: m_curNamespace->classList.append(m_curString); break;
         case InFile:      m_curFile->namespaceList.append(m_curString); break;
         case InGroup:     m_curGroup->namespaceList.append(m_curString); break;
-        default:   warn("Unexpected tag 'namespace' found"); break; 
+        default:   warn("Unexpected tag 'namespace' found"); break;
       }
     }
 
@@ -538,7 +557,7 @@ class TagFileParser : public QXmlDefaultHandler
       {
         case InGroup:      m_curGroup->fileList.append(m_curString); break;
         case InDir:        m_curDir->fileList.append(m_curString); break;
-        default:   warn("Unexpected tag 'file' found"); break; 
+        default:   warn("Unexpected tag 'file' found"); break;
       }
     }
 
@@ -547,7 +566,7 @@ class TagFileParser : public QXmlDefaultHandler
       switch(m_state)
       {
         case InGroup:      m_curGroup->fileList.append(m_curString); break;
-        default:   warn("Unexpected tag 'page' found"); break; 
+        default:   warn("Unexpected tag 'page' found"); break;
       }
     }
 
@@ -556,7 +575,7 @@ class TagFileParser : public QXmlDefaultHandler
       switch(m_state)
       {
         case InDir:      m_curDir->subdirList.append(m_curString); break;
-        default:   warn("Unexpected tag 'dir' found"); break; 
+        default:   warn("Unexpected tag 'dir' found"); break;
       }
     }
 
@@ -576,7 +595,7 @@ class TagFileParser : public QXmlDefaultHandler
     {
       if (m_state==InMember)
       {
-        m_curMember->type = m_curString; 
+        m_curMember->type = m_curString;
       }
       else
       {
@@ -596,7 +615,7 @@ class TagFileParser : public QXmlDefaultHandler
         case InDir:       m_curDir->name       = m_curString; break;
         case InMember:    m_curMember->name    = m_curString; break;
         case InPackage:   m_curPackage->name   = m_curString; break;
-        default: warn("Unexpected tag 'name' found"); break; 
+        default: warn("Unexpected tag 'name' found"); break;
       }
     }
 
@@ -621,12 +640,7 @@ class TagFileParser : public QXmlDefaultHandler
         {
           virt = Virtual;
         }
-        if (m_curClass->bases==0) 
-        {
-          m_curClass->bases = new QList<BaseInfo>;
-          m_curClass->bases->setAutoDelete(TRUE);
-        }
-        m_curClass->bases->append(new BaseInfo(m_curString,prot,virt));
+        m_curClass->bases.push_back(BaseInfo(m_curString,prot,virt));
       }
       else
       {
@@ -638,7 +652,7 @@ class TagFileParser : public QXmlDefaultHandler
     {
       if (m_state==InClass && m_curClass)
       {
-        m_curClass->bases->getLast()->name = m_curString;
+        m_curClass->bases.back().name = m_curString;
       }
       else
       {
@@ -673,7 +687,7 @@ class TagFileParser : public QXmlDefaultHandler
     {
       if (m_state==InClass && m_curClass)
       {
-        if (m_curClass->templateArguments==0) 
+        if (m_curClass->templateArguments==0)
         {
           m_curClass->templateArguments = new QList<QCString>;
           m_curClass->templateArguments->setAutoDelete(TRUE);
@@ -697,7 +711,7 @@ class TagFileParser : public QXmlDefaultHandler
         case InPage:      m_curPage->filename      = m_curString;    break;
         case InPackage:   m_curPackage->filename   = m_curString;    break;
         case InDir:       m_curDir->filename       = m_curString;    break;
-        default: warn("Unexpected tag 'filename' found"); break; 
+        default: warn("Unexpected tag 'filename' found"); break;
       }
     }
 
@@ -707,19 +721,19 @@ class TagFileParser : public QXmlDefaultHandler
       {
         case InFile:      m_curFile->path          = m_curString;    break;
         case InDir:       m_curDir->path           = m_curString;    break;
-        default: warn("Unexpected tag 'path' found");     break; 
+        default: warn("Unexpected tag 'path' found");     break;
       }
     }
-    
+
     void endAnchor()
     {
       if (m_state==InMember)
       {
-        m_curMember->anchor = m_curString; 
+        m_curMember->anchor = m_curString;
       }
       else if (m_state==InClass)
       {
-        m_curClass->anchor = m_curString; 
+        m_curClass->anchor = m_curString;
       }
       else
       {
@@ -731,7 +745,7 @@ class TagFileParser : public QXmlDefaultHandler
     {
       if (m_state==InMember)
       {
-        m_curMember->clangId = m_curString; 
+        m_curMember->clangId = m_curString;
       }
       else if (m_state==InClass)
       {
@@ -748,24 +762,24 @@ class TagFileParser : public QXmlDefaultHandler
     }
 
 
-    
+
     void endAnchorFile()
     {
       if (m_state==InMember)
       {
-        m_curMember->anchorFile = m_curString; 
+        m_curMember->anchorFile = m_curString;
       }
       else
       {
         warn("Unexpected tag 'anchorfile' found");
       }
     }
-    
+
     void endArglist()
     {
       if (m_state==InMember)
       {
-        m_curMember->arglist = m_curString; 
+        m_curMember->arglist = m_curString;
       }
       else
       {
@@ -778,7 +792,7 @@ class TagFileParser : public QXmlDefaultHandler
       {
         case InGroup:     m_curGroup->title     = m_curString;    break;
         case InPage:      m_curPage->title      = m_curString;    break;
-        default: warn("Unexpected tag 'title' found"); break; 
+        default: warn("Unexpected tag 'title' found"); break;
       }
     }
 
@@ -874,7 +888,7 @@ class TagFileParser : public QXmlDefaultHandler
       return TRUE;
     }
 
-    bool startElement( const QString&, const QString&, 
+    bool startElement( const QString&, const QString&,
                        const QString&name, const QXmlAttributes& attrib )
     {
       //printf("startElement '%s'\n",name.data());
@@ -883,7 +897,7 @@ class TagFileParser : public QXmlDefaultHandler
       {
         (*handler)(attrib);
       }
-      else 
+      else
       {
         warn("Unknown tag '%s' found!",name.data());
       }
@@ -898,26 +912,26 @@ class TagFileParser : public QXmlDefaultHandler
       {
         (*handler)();
       }
-      else 
+      else
       {
         warn("Unknown tag '%s' found!",name.data());
       }
       return TRUE;
     }
 
-    bool characters ( const QString & ch ) 
+    bool characters ( const QString & ch )
     {
       m_curString+=ch.utf8();
       return TRUE;
     }
 
     void dump();
-    void buildLists(Entry *root);
+    void buildLists(const std::shared_ptr<Entry> &root);
     void addIncludes();
-    
+
   private:
-    void buildMemberList(Entry *ce,QList<TagMemberInfo> &members);
-    void addDocAnchors(Entry *e,const TagAnchorInfoList &l);
+    void buildMemberList(const std::shared_ptr<Entry> &ce,QList<TagMemberInfo> &members);
+    void addDocAnchors(const std::shared_ptr<Entry> &e,const TagAnchorInfoList &l);
     QList<TagClassInfo>        m_tagFileClasses;
     QList<TagFileInfo>         m_tagFileFiles;
     QList<TagNamespaceInfo>    m_tagFileNamespaces;
@@ -947,7 +961,7 @@ class TagFileParser : public QXmlDefaultHandler
     QCString                   m_inputFileName;
 };
 
-/** Error handler for the XML tag file parser. 
+/** Error handler for the XML tag file parser.
  *
  *  Basically dumps all fatal error to stderr using err().
  */
@@ -988,14 +1002,9 @@ void TagFileParser::dump()
   {
     msg("class '%s'\n",cd->name.data());
     msg("  filename '%s'\n",cd->filename.data());
-    if (cd->bases)
+    for (const BaseInfo &bi : cd->bases)
     {
-      QListIterator<BaseInfo> bii(*cd->bases);
-      BaseInfo *bi;
-      for ( bii.toFirst() ; (bi=bii.current()) ; ++bii) 
-      {
-        msg( "  base: %s \n", bi->name.data() );
-      }
+      msg( "  base: %s \n", bi.name.data() );
     }
 
     QListIterator<TagMemberInfo> mci(cd->members);
@@ -1017,8 +1026,8 @@ void TagFileParser::dump()
     msg("namespace '%s'\n",nd->name.data());
     msg("  filename '%s'\n",nd->filename.data());
     QCStringList::Iterator it;
-    for ( it = nd->classList.begin(); 
-        it != nd->classList.end(); ++it ) 
+    for ( it = nd->classList.begin();
+        it != nd->classList.end(); ++it )
     {
       msg( "  class: %s \n", (*it).data() );
     }
@@ -1042,13 +1051,13 @@ void TagFileParser::dump()
     msg("file '%s'\n",fd->name.data());
     msg("  filename '%s'\n",fd->filename.data());
     QCStringList::Iterator it;
-    for ( it = fd->namespaceList.begin(); 
-        it != fd->namespaceList.end(); ++it ) 
+    for ( it = fd->namespaceList.begin();
+        it != fd->namespaceList.end(); ++it )
     {
       msg( "  namespace: %s \n", (*it).data() );
     }
-    for ( it = fd->classList.begin(); 
-        it != fd->classList.end(); ++it ) 
+    for ( it = fd->classList.begin();
+        it != fd->classList.end(); ++it )
     {
       msg( "  class: %s \n", (*it).data() );
     }
@@ -1080,28 +1089,28 @@ void TagFileParser::dump()
     msg("group '%s'\n",gd->name.data());
     msg("  filename '%s'\n",gd->filename.data());
     QCStringList::Iterator it;
-    for ( it = gd->namespaceList.begin(); 
-        it != gd->namespaceList.end(); ++it ) 
+    for ( it = gd->namespaceList.begin();
+        it != gd->namespaceList.end(); ++it )
     {
       msg( "  namespace: %s \n", (*it).data() );
     }
-    for ( it = gd->classList.begin(); 
-        it != gd->classList.end(); ++it ) 
+    for ( it = gd->classList.begin();
+        it != gd->classList.end(); ++it )
     {
       msg( "  class: %s \n", (*it).data() );
     }
-    for ( it = gd->fileList.begin(); 
-        it != gd->fileList.end(); ++it ) 
+    for ( it = gd->fileList.begin();
+        it != gd->fileList.end(); ++it )
     {
       msg( "  file: %s \n", (*it).data() );
     }
-    for ( it = gd->subgroupList.begin(); 
-        it != gd->subgroupList.end(); ++it ) 
+    for ( it = gd->subgroupList.begin();
+        it != gd->subgroupList.end(); ++it )
     {
       msg( "  subgroup: %s \n", (*it).data() );
     }
-    for ( it = gd->pageList.begin(); 
-        it != gd->pageList.end(); ++it ) 
+    for ( it = gd->pageList.begin();
+        it != gd->pageList.end(); ++it )
     {
       msg( "  page: %s \n", (*it).data() );
     }
@@ -1134,33 +1143,33 @@ void TagFileParser::dump()
     msg("dir '%s'\n",dd->name.data());
     msg("  path '%s'\n",dd->path.data());
     QCStringList::Iterator it;
-    for ( it = dd->fileList.begin(); 
-        it != dd->fileList.end(); ++it ) 
+    for ( it = dd->fileList.begin();
+        it != dd->fileList.end(); ++it )
     {
       msg( "  file: %s \n", (*it).data() );
     }
-    for ( it = dd->subdirList.begin(); 
-        it != dd->subdirList.end(); ++it ) 
+    for ( it = dd->subdirList.begin();
+        it != dd->subdirList.end(); ++it )
     {
       msg( "  subdir: %s \n", (*it).data() );
     }
   }
 }
 
-void TagFileParser::addDocAnchors(Entry *e,const TagAnchorInfoList &l)
+void TagFileParser::addDocAnchors(const std::shared_ptr<Entry> &e,const TagAnchorInfoList &l)
 {
   QListIterator<TagAnchorInfo> tli(l);
   TagAnchorInfo *ta;
   for (tli.toFirst();(ta=tli.current());++tli)
   {
-    if (Doxygen::sectionDict->find(ta->label)==0)
+    if (SectionManager::instance().find(ta->label)==0)
     {
       //printf("New sectionInfo file=%s anchor=%s\n",
       //    ta->fileName.data(),ta->label.data());
-      SectionInfo *si=new SectionInfo(ta->fileName,-1,ta->label,ta->title,
-          SectionInfo::Anchor,0,m_tagName);
-      Doxygen::sectionDict->append(ta->label,si);
-      e->anchors->append(si);
+      SectionInfo *si=SectionManager::instance().add(
+          ta->label,ta->fileName,-1,ta->title,
+          SectionType::Anchor,0,m_tagName);
+      e->anchors.push_back(si);
     }
     else
     {
@@ -1169,21 +1178,19 @@ void TagFileParser::addDocAnchors(Entry *e,const TagAnchorInfoList &l)
   }
 }
 
-void TagFileParser::buildMemberList(Entry *ce,QList<TagMemberInfo> &members)
+void TagFileParser::buildMemberList(const std::shared_ptr<Entry> &ce,QList<TagMemberInfo> &members)
 {
   QListIterator<TagMemberInfo> mii(members);
   TagMemberInfo *tmi;
   for (;(tmi=mii.current());++mii)
   {
-    Entry *me      = new Entry;
+    std::shared_ptr<Entry> me = std::make_shared<Entry>();
     me->type       = tmi->type;
     me->name       = tmi->name;
     me->args       = tmi->arglist;
     if (!me->args.isEmpty())
     {
-      delete me->argList;
-      me->argList = new ArgumentList;
-      stringToArgumentList(me->args,me->argList);
+      stringToArgumentList(SrcLangExt_Cpp,me->args,me->argList);
     }
     if (tmi->enumValues.count()>0)
     {
@@ -1192,17 +1199,16 @@ void TagFileParser::buildMemberList(Entry *ce,QList<TagMemberInfo> &members)
       TagEnumValueInfo *evi;
       for (evii.toFirst();(evi=evii.current());++evii)
       {
-        Entry *ev      = new Entry;
+        std::shared_ptr<Entry> ev = std::make_shared<Entry>();
         ev->type       = "@";
         ev->name       = evi->name;
         ev->id         = evi->clangid;
         ev->section    = Entry::VARIABLE_SEC;
-        TagInfo *ti    = new TagInfo;
-        ti->tagName    = m_tagName;
-        ti->anchor     = evi->anchor;
-        ti->fileName   = evi->file;
-        ev->tagInfo    = ti;
-        me->addSubEntry(ev);
+        ev->tagInfoData.tagName    = m_tagName;
+        ev->tagInfoData.anchor     = evi->anchor;
+        ev->tagInfoData.fileName   = evi->file;
+        ev->hasTagInfo    = TRUE;
+        me->moveToSubEntryAndKeep(ev);
       }
     }
     me->protection = tmi->prot;
@@ -1212,14 +1218,13 @@ void TagFileParser::buildMemberList(Entry *ce,QList<TagMemberInfo> &members)
     me->id         = tmi->clangId;
     if (ce->section == Entry::GROUPDOC_SEC)
     {
-      me->groups->append(new Grouping(ce->name,Grouping::GROUPING_INGROUP));
+      me->groups.push_back(Grouping(ce->name,Grouping::GROUPING_INGROUP));
     }
     addDocAnchors(me,tmi->docAnchors);
-    TagInfo *ti    = new TagInfo;
-    ti->tagName    = m_tagName;
-    ti->anchor     = tmi->anchor;
-    ti->fileName   = tmi->anchorFile;
-    me->tagInfo    = ti;
+    me->tagInfoData.tagName    = m_tagName;
+    me->tagInfoData.anchor     = tmi->anchor;
+    me->tagInfoData.fileName   = tmi->anchorFile;
+    me->hasTagInfo    = TRUE;
     if (tmi->kind=="define")
     {
       me->type="#define";
@@ -1287,7 +1292,7 @@ void TagFileParser::buildMemberList(Entry *ce,QList<TagMemberInfo> &members)
       me->section = Entry::FUNCTION_SEC;
       me->mtype = Slot;
     }
-    ce->addSubEntry(me);
+    ce->moveToSubEntryAndKeep(me);
   }
 }
 
@@ -1305,17 +1310,17 @@ static QCString stripPath(const QCString &s)
 }
 
 /*! Injects the info gathered by the XML parser into the Entry tree.
- *  This tree contains the information extracted from the input in a 
+ *  This tree contains the information extracted from the input in a
  *  "unrelated" form.
  */
-void TagFileParser::buildLists(Entry *root)
+void TagFileParser::buildLists(const std::shared_ptr<Entry> &root)
 {
   // build class list
   QListIterator<TagClassInfo> cit(m_tagFileClasses);
   TagClassInfo *tci;
   for (cit.toFirst();(tci=cit.current());++cit)
   {
-    Entry *ce = new Entry;
+    std::shared_ptr<Entry> ce = std::make_shared<Entry>();
     ce->section = Entry::CLASS_SEC;
     switch (tci->kind)
     {
@@ -1339,42 +1344,32 @@ void TagFileParser::buildLists(Entry *root)
       ce->name+="-p";
     }
     addDocAnchors(ce,tci->docAnchors);
-    TagInfo *ti  = new TagInfo;
-    ti->tagName  = m_tagName;
-    ti->anchor   = tci->anchor;
-    ti->fileName = tci->filename;
+    ce->tagInfoData.tagName  = m_tagName;
+    ce->tagInfoData.anchor   = tci->anchor;
+    ce->tagInfoData.fileName = tci->filename;
+    ce->hasTagInfo  = TRUE;
     ce->id       = tci->clangId;
-    ce->tagInfo  = ti;
     ce->lang     = tci->isObjC ? SrcLangExt_ObjC : SrcLangExt_Unknown;
     // transfer base class list
-    if (tci->bases)
-    {
-      delete ce->extends;
-      ce->extends = tci->bases; tci->bases = 0;
-    }
+    ce->extends  = tci->bases;
+    tci->bases.clear();
     if (tci->templateArguments)
     {
-      if (ce->tArgLists==0)
-      {
-        ce->tArgLists = new QList<ArgumentList>;
-        ce->tArgLists->setAutoDelete(TRUE);
-      }
-      ArgumentList *al = new ArgumentList;
-      ce->tArgLists->append(al);
-
+      ArgumentList al;
       QListIterator<QCString> sli(*tci->templateArguments);
       QCString *argName;
       for (;(argName=sli.current());++sli)
       {
-        Argument *a = new Argument;
-        a->type = "class";
-        a->name = *argName;
-        al->append(a);
+        Argument a;
+        a.type = "class";
+        a.name = *argName;
+        al.push_back(a);
       }
+      ce->tArgLists.push_back(al);
     }
 
     buildMemberList(ce,tci->members);
-    root->addSubEntry(ce);
+    root->moveToSubEntryAndKeep(ce);
   }
 
   // build file list
@@ -1382,36 +1377,32 @@ void TagFileParser::buildLists(Entry *root)
   TagFileInfo *tfi;
   for (fit.toFirst();(tfi=fit.current());++fit)
   {
-    Entry *fe = new Entry;
+    std::shared_ptr<Entry> fe = std::make_shared<Entry>();
     fe->section = guessSection(tfi->name);
     fe->name     = tfi->name;
     addDocAnchors(fe,tfi->docAnchors);
-    TagInfo *ti  = new TagInfo;
-    ti->tagName  = m_tagName;
-    ti->fileName = tfi->filename;
-    fe->tagInfo  = ti;
-    
+    fe->tagInfoData.tagName  = m_tagName;
+    fe->tagInfoData.fileName = tfi->filename;
+    fe->hasTagInfo = TRUE;
+
     QCString fullName = m_tagName+":"+tfi->path+stripPath(tfi->name);
     fe->fileName = fullName;
     //printf("createFileDef() filename=%s\n",tfi->filename.data());
-    FileDef *fd = createFileDef(m_tagName+":"+tfi->path,
+    std::unique_ptr<FileDef> fd { createFileDef(m_tagName+":"+tfi->path,
                               tfi->name,m_tagName,
-                              tfi->filename
-                             );
+                              tfi->filename) };
     FileName *mn;
-    if ((mn=Doxygen::inputNameDict->find(tfi->name)))
+    if ((mn=Doxygen::inputNameLinkedMap->find(tfi->name)))
     {
-      mn->append(fd);
+      mn->push_back(std::move(fd));
     }
     else
     {
-      mn = new FileName(fullName,tfi->name);
-      mn->append(fd);
-      Doxygen::inputNameList->inSort(mn);
-      Doxygen::inputNameDict->insert(tfi->name,mn);
+      mn = Doxygen::inputNameLinkedMap->add(tfi->name,fullName);
+      mn->push_back(std::move(fd));
     }
     buildMemberList(fe,tfi->members);
-    root->addSubEntry(fe);
+    root->moveToSubEntryAndKeep(fe);
   }
 
   // build namespace list
@@ -1419,18 +1410,17 @@ void TagFileParser::buildLists(Entry *root)
   TagNamespaceInfo *tni;
   for (nit.toFirst();(tni=nit.current());++nit)
   {
-    Entry *ne    = new Entry;
+    std::shared_ptr<Entry> ne = std::make_shared<Entry>();
     ne->section  = Entry::NAMESPACE_SEC;
     ne->name     = tni->name;
     addDocAnchors(ne,tni->docAnchors);
-    TagInfo *ti  = new TagInfo;
-    ti->tagName  = m_tagName;
-    ti->fileName = tni->filename;
+    ne->tagInfoData.tagName  = m_tagName;
+    ne->tagInfoData.fileName = tni->filename;
+    ne->hasTagInfo  = TRUE;
     ne->id       = tni->clangId;
-    ne->tagInfo  = ti;
 
     buildMemberList(ne,tni->members);
-    root->addSubEntry(ne);
+    root->moveToSubEntryAndKeep(ne);
   }
 
   // build package list
@@ -1438,17 +1428,16 @@ void TagFileParser::buildLists(Entry *root)
   TagPackageInfo *tpgi;
   for (pit.toFirst();(tpgi=pit.current());++pit)
   {
-    Entry *pe    = new Entry;
+    std::shared_ptr<Entry> pe = std::make_shared<Entry>();
     pe->section  = Entry::PACKAGE_SEC;
     pe->name     = tpgi->name;
     addDocAnchors(pe,tpgi->docAnchors);
-    TagInfo *ti  = new TagInfo;
-    ti->tagName  = m_tagName;
-    ti->fileName = tpgi->filename;
-    pe->tagInfo  = ti;
+    pe->tagInfoData.tagName  = m_tagName;
+    pe->tagInfoData.fileName = tpgi->filename;
+    pe->hasTagInfo  = TRUE;
 
     buildMemberList(pe,tpgi->members);
-    root->addSubEntry(pe);
+    root->moveToSubEntryAndKeep(pe);
   }
 
   // build group list
@@ -1456,18 +1445,17 @@ void TagFileParser::buildLists(Entry *root)
   TagGroupInfo *tgi;
   for (git.toFirst();(tgi=git.current());++git)
   {
-    Entry *ge    = new Entry;
+    std::shared_ptr<Entry> ge = std::make_shared<Entry>();
     ge->section  = Entry::GROUPDOC_SEC;
     ge->name     = tgi->name;
     ge->type     = tgi->title;
     addDocAnchors(ge,tgi->docAnchors);
-    TagInfo *ti  = new TagInfo;
-    ti->tagName  = m_tagName;
-    ti->fileName = tgi->filename;
-    ge->tagInfo  = ti;
+    ge->tagInfoData.tagName  = m_tagName;
+    ge->tagInfoData.fileName = tgi->filename;
+    ge->hasTagInfo  = TRUE;
 
     buildMemberList(ge,tgi->members);
-    root->addSubEntry(ge);
+    root->moveToSubEntryAndKeep(ge);
   }
 
   // set subgroup relations bug_774118
@@ -1476,13 +1464,16 @@ void TagFileParser::buildLists(Entry *root)
     QCStringList::Iterator it;
     for ( it = tgi->subgroupList.begin(); it != tgi->subgroupList.end(); ++it )
     {
-      QListIterator<Entry> eli(*(root->children()));
-      Entry *childNode;
-      for (eli.toFirst();(childNode=eli.current());++eli)
+      //QListIterator<Entry> eli(*(root->children()));
+      //Entry *childNode;
+      //for (eli.toFirst();(childNode=eli.current());++eli)
+      const auto &children = root->children();
+      auto i = std::find_if(children.begin(),children.end(),
+          [&](const std::shared_ptr<Entry> &e) { return e->name = *it; });
+      if (i!=children.end())
       {
-        if (childNode->name == (*it)) break;
+        (*i)->groups.push_back(Grouping(tgi->name,Grouping::GROUPING_INGROUP));
       }
-      childNode->groups->append(new Grouping(tgi->name,Grouping::GROUPING_INGROUP));
     }
   }
 
@@ -1491,16 +1482,16 @@ void TagFileParser::buildLists(Entry *root)
   TagPageInfo *tpi;
   for (pgit.toFirst();(tpi=pgit.current());++pgit)
   {
-    Entry *pe    = new Entry;
-    pe->section  = tpi->filename=="index" ? Entry::MAINPAGEDOC_SEC : Entry::PAGEDOC_SEC;
+    std::shared_ptr<Entry> pe = std::make_shared<Entry>();
+    bool isIndex = (stripExtensionGeneral(tpi->filename,getFileNameExtension(tpi->filename))=="index");
+    pe->section  = isIndex ? Entry::MAINPAGEDOC_SEC : Entry::PAGEDOC_SEC;
     pe->name     = tpi->name;
     pe->args     = tpi->title;
     addDocAnchors(pe,tpi->docAnchors);
-    TagInfo *ti  = new TagInfo;
-    ti->tagName  = m_tagName;
-    ti->fileName = tpi->filename;
-    pe->tagInfo  = ti;
-    root->addSubEntry(pe);
+    pe->tagInfoData.tagName  = m_tagName;
+    pe->tagInfoData.fileName = tpi->filename;
+    pe->hasTagInfo  = TRUE;
+    root->moveToSubEntryAndKeep(pe);
   }
 }
 
@@ -1511,13 +1502,10 @@ void TagFileParser::addIncludes()
   for (fit.toFirst();(tfi=fit.current());++fit)
   {
     //printf("tag file tagName=%s path=%s name=%s\n",m_tagName.data(),tfi->path.data(),tfi->name.data());
-    FileName *fn = Doxygen::inputNameDict->find(tfi->name);
+    FileName *fn = Doxygen::inputNameLinkedMap->find(tfi->name);
     if (fn)
     {
-      //printf("found\n");
-      FileNameIterator fni(*fn);
-      FileDef *fd;
-      for (;(fd=fni.current());++fni)
+      for (const auto &fd : *fn)
       {
         //printf("input file path=%s name=%s\n",fd->getPath().data(),fd->name().data());
         if (fd->getPath()==QCString(m_tagName+":"+tfi->path))
@@ -1528,19 +1516,17 @@ void TagFileParser::addIncludes()
           for (;(ii=mii.current());++mii)
           {
             //printf("ii->name='%s'\n",ii->name.data());
-            FileName *ifn = Doxygen::inputNameDict->find(ii->name);
+            FileName *ifn = Doxygen::inputNameLinkedMap->find(ii->name);
             ASSERT(ifn!=0);
             if (ifn)
             {
-              FileNameIterator ifni(*ifn);
-              FileDef *ifd;
-              for (;(ifd=ifni.current());++ifni)
+              for (const auto &ifd : *ifn)
               {
                 //printf("ifd->getOutputFileBase()=%s ii->id=%s\n",
                 //        ifd->getOutputFileBase().data(),ii->id.data());
                 if (ifd->getOutputFileBase()==QCString(ii->id))
                 {
-                  fd->addIncludeDependency(ifd,ii->text,ii->isLocal,ii->isImported,FALSE);
+                  fd->addIncludeDependency(ifd.get(),ii->text,ii->isLocal,ii->isImported,FALSE);
                 }
               }
             }
@@ -1551,7 +1537,7 @@ void TagFileParser::addIncludes()
   }
 }
 
-void parseTagFile(Entry *root,const char *fullName)
+void parseTagFile(const std::shared_ptr<Entry> &root,const char *fullName)
 {
   QFileInfo fi(fullName);
   if (!fi.exists()) return;
@@ -1568,5 +1554,3 @@ void parseTagFile(Entry *root,const char *fullName)
   handler.addIncludes();
   //handler.dump();
 }
-
-
